#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor
# chapter 13, example 4.
#
# Time counter engine.
# The output from an time counter engine is used to control
# horizontal and vertical motion of a figure object.
# The resulting effect is that the figure jumps across
# the screen.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])  
    if myWindow == None: sys.exit(1)     

    root = SoSeparator()
    
    # Add a camera and light
    myCamera = SoPerspectiveCamera()
    myCamera.position = (-8.0, -7.0, 20.0)
    myCamera.heightAngle = M_PI/2.5
    myCamera.nearDistance = 15.0
    myCamera.farDistance = 25.0
    root.addChild(myCamera)
    root.addChild(SoDirectionalLight())

##############################################################
# CODE FOR The Inventor Mentor STARTS HERE

    # Set up transformations
    jumpTranslation = SoTranslation()
    root.addChild(jumpTranslation)
    initialTransform = SoTransform()
    initialTransform.translation = (-20., 0., 0.)
    initialTransform.scaleFactor = (40., 40., 40.)
    initialTransform.rotation.setValue(SbVec3f(1,0,0), M_PI/2.)
    root.addChild(initialTransform)

    # Read the man object from a file and add to the scene
    myInput = SoInput()
    if not myInput.openFile("jumpyMan.iv"):
        sys.exit(1)
    manObject = SoDB.readAll(myInput)
    if manObject == None:
        sys.exit(1)
    root.addChild(manObject)

    # Create two counters, and connect to X and Y translations.
    # The Y counter is small and high frequency.
    # The X counter is large and low frequency.
    # This results in small jumps across the screen, 
    # left to right, again and again and again and ....
    jumpHeightCounter = SoTimeCounter()
    jumpWidthCounter = SoTimeCounter()
    jump = SoComposeVec3f()

    jumpHeightCounter.max = 4
    jumpHeightCounter.frequency = 1.5
    jumpWidthCounter.max = 40
    jumpWidthCounter.frequency = 0.15

    jump.x.connectFrom(jumpWidthCounter.output)
    jump.y.connectFrom(jumpHeightCounter.output)
    jumpTranslation.translation.connectFrom(jump.vector)

# CODE FOR The Inventor Mentor ENDS HERE
##############################################################

    myRenderArea = SoGuiRenderArea(myWindow)
    myRegion = SbViewportRegion(myRenderArea.getSize()) 
    myRenderArea.setSceneGraph(root)
    myRenderArea.setTitle("Jumping Man")
    myRenderArea.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 3, example 2.
#
# This code shows how to create a robot out of various nodes.
# It introduces shared instancing of nodes to create two legs
# using two instances of the same subgraph.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def makeRobot():
##############################################################
# CODE FOR The Inventor Mentor STARTS HERE
    # Robot with legs

    # Construct parts for legs (thigh, calf and foot)
    thigh = SoCube()
    thigh.width = 1.2
    thigh.height = 2.2
    thigh.depth = 1.1
    
    calfTransform = SoTransform()
    calfTransform.translation.setValue(0, -2.25, 0.0)
    
    calf = SoCube()
    calf.width, calf.height, calf.depth = 1, 2.2, 1

    footTransform = SoTransform()
    footTransform.translation = (0, -1.5, .5)

    foot = SoCube()
    foot.width, foot.height, foot.depth = 0.8, 0.8, 2

    # Put leg parts together
    leg = SoGroup()
    leg.addChild(thigh)
    leg.addChild(calfTransform)
    leg.addChild(calf)
    leg.addChild(footTransform)
    leg.addChild(foot)
    
    leftTransform = SoTransform()
    leftTransform.translation = (1, -4.25, 0)
    
    # Left leg
    leftLeg = SoSeparator()
    leftLeg.addChild(leftTransform)
    leftLeg.addChild(leg)
    
    rightTransform = SoTransform()
    rightTransform.translation = (-1, -4.25, 0)
    
    # Right leg
    rightLeg = SoSeparator()
    rightLeg.addChild(rightTransform)
    rightLeg.addChild(leg)
    
    # Parts for body
    bodyTransform = SoTransform()
    bodyTransform.translation = (0.0, 3.0, 0.0)
    
    bronze = SoMaterial()
    bronze.ambientColor = (.33, .22, .27)
    bronze.diffuseColor = (.78, .57, .11)
    bronze.specularColor = (.99, .94, .81)
    bronze.shininess = .28
    
    bodyCylinder = SoCylinder()
    bodyCylinder.radius = 2.5
    bodyCylinder.height = 6
    
    # Construct body out of parts 
    body = SoSeparator()
    body.addChild(bodyTransform)      
    body.addChild(bronze)
    body.addChild(bodyCylinder)
    body.addChild(leftLeg)
    body.addChild(rightLeg)
    
    # Head parts
    headTransform = SoTransform()
    headTransform.translation = (0, 7.5, 0)
    headTransform.scaleFactor = (1.5, 1.5, 1.5)
    
    silver = SoMaterial()
    silver.ambientColor = (.2, .2, .2)
    silver.diffuseColor = (.6, .6, .6)
    silver.specularColor = (.5, .5, .5)
    silver.shininess = .5
    
    headSphere = SoSphere()
    
    # Construct head
    head = SoSeparator()
    head.addChild(headTransform)
    head.addChild(silver)
    head.addChild(headSphere)
    
    # Robot is just head and body
    robot = SoSeparator()
    robot.addChild(body)               
    robot.addChild(head)
    
# CODE FOR The Inventor Mentor ENDS HERE
##############################################################

    return robot


def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoSeparator()
    
    # This function contains our code fragment.
    root.addChild(makeRobot())
    
    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Robot")
    myViewer.show()
    myViewer.viewAll()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2009 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# Pivy distutils setup script.
#

"""Pivy is a Coin binding for Python. Coin is a high-level 3D graphics
library with a C++ Application Programming Interface. Coin uses
scene-graph data structures to render real-time graphics suitable for
mostly all kinds of scientific and engineering visualization
applications.
"""
from __future__ import print_function

###
# Setup file for the Pivy distribution.
#
import glob, os, shutil, subprocess, sys

from distutils.command.build import build
from distutils.command.clean import clean
from distutils.command.install import install
from distutils.core import setup
from distutils.extension import Extension
from distutils import sysconfig

# if we are on a Gentoo box salute the chap and output stuff in nice colors
# Gentoo is Python friendly, so be especially friendly to them! ;)
try:
    from portage.output import green, blue, turquoise, red, yellow
    print(red("Oooh, it's a Gentoo! Nice nice! tuhtah salutes you! :)"))
except:
    try:
        from colorama import Fore, Style
        def red(text): return Fore.RED + text + Style.RESET_ALL
        def green(text): return Fore.GREEN + text + Style.RESET_ALL
        def blue(text): return Fore.BLUE + text + Style.RESET_ALL
        def turquoise(text): return Fore.CYAN + text + Style.RESET_ALL
        def yellow(text): return Fore.YELLOW + text + Style.RESET_ALL
    except:
        def red(text): return text
        def green(text): return text
        def blue(text): return text
        def turquoise(text): return text
        def yellow(text): return text

PIVY_CLASSIFIERS = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: Unix
Programming Language :: Python
Topic :: Multimedia :: Graphics
Topic :: Multimedia :: Graphics :: 3D Modeling
Topic :: Multimedia :: Graphics :: 3D Rendering
Topic :: Software Development :: Libraries :: Python Modules
"""

__dir_name__ = os.path.dirname(__file__)
pivy_dir = os.path.join(__dir_name__, "pivy")
sys.path.append(pivy_dir)
import pivy_meta
PIVY_VERSION = pivy_meta.__version__
sys.path.pop(-1)

class pivy_build(build):
    PIVY_SNAKES = r"""
                            _____
                        .-'`     '.                                     
                     __/  __       \                                   
                    /  \ /  \       |    ___                          
                   | /`\| /`\|      | .-'  /^\/^\                   
                   | \(/| \(/|      |/     |) |)|                     
                  .-\__/ \__/       |      \_/\_/__..._             
          _...---'-.                /   _              '.               
         /,      ,             \   '|  `\                \           
        | ))     ))           /`|   \    `.       /)  /) |             
        | `      `          .'       |     `-._         /               
        \                 .'         |     ,_  `--....-'               
         `.           __.' ,         |     / /`'''`                     
           `'-.____.-' /  /,         |    / /                           
               `. `-.-` .'  \        /   / |                           
                 `-.__.'|    \      |   |  |-.                         
                    _.._|     |     /   |  |  `'.                       
              .-''``    |     |     |   /  |     `-.                    
           .'`         /      /     /  |   |        '.                  
         /`           /      /     |   /   |\         \               
        /            |      |      |   |   /\          |               
       ||            |      /      |   /     '.        |                
       |\            \      |      /   |       '.      /              
       \ `.           '.    /      |    \        '---'/               
        \  '.           `-./        \    '.          /                
         '.  `'.            `-._     '.__  '-._____.'--'''''--.         
           '-.  `'--._          `.__     `';----`              \       
              `-.     `-.          `.''```                     ;        
                 `'-..,_ `-.         `'-.                     /         
                        '.  '.           '.                 .'          


                            ~~~ HISSSSSSSSSS ~~~
                           Welcome to Pivy %s!
                 Building Pivy has never been so much fun!

    """ % PIVY_VERSION

    pivy_header_include = """\
#ifdef __PIVY__
%%include %s
#endif

"""
    
    SWIG = ((sys.platform == "win32" and "swig.exe") or "swig")

    SWIG_SUPPRESS_WARNINGS = "-w302,306,307,312,314,325,361,362,467,389,503,509,510"
    if sys.version_info.major < 3:
        SWIG_PARAMS = "-c++ -python -includeall -modern -D__PIVY__ " + \
                      "-I. -Ifake_headers -I\"%s\" %s -o %s_wrap.cpp " + \
                      "interfaces" + os.sep + "%s.i"
    else:
        SWIG_PARAMS = "-c++ -python -includeall -modern -py3 -D__PIVY__ " + \
                      "-I. -Ifake_headers -I\"%s\" %s -o %s_wrap.cpp " + \
                      "interfaces" + os.sep + "%s.i"


    SOGUI = ['soqt', 'soxt', 'sogtk', 'sowin']

    MODULES = {}
    MODULES['coin']      = ['_coin',      'coin-config',      'pivy.',        "coin"]
    MODULES['soqt']      = ['gui._soqt',  'soqt-config',      'pivy.gui.',    "soqt"]

    if sys.version_info.major < 3:
        MODULES['simvoleon'] = ['_simvoleon', 'simvoleon-config', 'pivy.',        "simvoleon"]
        MODULES['soxt']      = ['gui._soxt',  'soxt-config',      'pivy.gui.',    "soxt"]
        MODULES['sogtk']     = ['gui._sogtk', 'sogtk-config',     'pivy.gui.',    "sogtk"]
        MODULES['sowin']     = ['gui._sowin', 'sowin-config',     'pivy.gui.',    "sowin"]

    if sys.version_info.major < 3:
        MODULES['coin'][3] = 'coin2'
        MODULES['soqt'][3] = 'soqt2'

    SUPPORTED_SWIG_VERSIONS = ['1.3.31', '1.3.33', '1.3.35', '1.3.40']
    SWIG_VERSION = ""
    SWIG_COND_SYMBOLS = []
    CXX_INCS = "-Iinterfaces "
    CXX_LIBS = ""

    ext_modules = []
    py_modules = ['pivy.quarter.ContextMenu',
                  'pivy.quarter.ImageReader',
                  'pivy.quarter.QuarterWidget',
                  'pivy.quarter.SensorManager',
                  'pivy.quarter.SignalThread',
                  'pivy.quarter.devices.DeviceHandler',
                  'pivy.quarter.devices.DeviceManager',
                  'pivy.quarter.devices.KeyboardHandler',
                  'pivy.quarter.devices.MouseHandler',
                  'pivy.quarter.eventhandlers.DragDropHandler',
                  'pivy.quarter.eventhandlers.EventHandler',
                  'pivy.quarter.eventhandlers.EventManager',
                  'pivy.quarter.plugins.designer.python.PyQuarterWidgetPlugin',
                  'pivy.utils',
                  'pivy.graphics.']

    if sys.version_info.major < 3:
        py_modules = ['pivy.sogui'] + py_modules

    def do_os_popen(self, cmd):
        "return the output of a command in a single line"
        fd = os.popen(cmd)
        lines = fd.readlines()
        for i in range(len(lines)):
            lines[i] = lines[i].strip()
        lines = " ".join(lines)
        fd.close()
        return lines

    def check_cmd_exists(self, cmd):
        "return the path of the specified command if it exists"
        print(blue("Checking for %s..." % cmd))
        for path in os.environ['PATH'].split(os.path.pathsep):
            if os.path.exists(os.path.join(path, cmd)):
                print(blue("'%s'" % os.path.join(path, cmd)))
                return 1
        print(red("not found."))
        return 0

    def check_python_version(self):
        "check the Python version"
        print(blue("Python version...%s" % sys.version.split(" ")[0]))
        if int(sys.version[0]) < 2:
            print(red("Pivy only works with Python versions >= 2.0."))
            sys.exit(1)

    def check_coin_version(self):
        "check the Coin version"
        if sys.platform == "win32": return
        if not self.check_cmd_exists("coin-config"):
            sys.exit(1)
        print(blue("Coin version..."))
        version = self.do_os_popen("coin-config --version")
        print(blue("%s" % version))
        if not version.startswith('3'):
            print(yellow("** Warning: Pivy has only been tested with Coin "
                         "versions Coin-dev 3."))

    def check_simvoleon_version(self):
        "return if SIMVoleon is available and check the version"
        if sys.platform == "win32" or not self.check_cmd_exists("simvoleon-config"):
            self.MODULES.pop('simvoleon', None)
            return False

        print(blue("SIMVoleon version..."))
        version = self.do_os_popen("simvoleon-config --version")
        print(blue("%s" % version))
        if not version.startswith('2.0'):
            print(yellow("** Warning: Pivy has only been tested with SIMVoleon "
                         "versions 2.0.x."))
        return True

    def check_gui_bindings(self):
        "check for availability of SoGui bindings and removes the not available ones"
        if sys.platform == "win32":
            self.MODULES.pop('soxt', None)
            self.MODULES.pop('sogtk', None)
            print(blue("Checking for SoWin..."))
            if not os.path.exists(os.path.join(os.getenv("COINDIR"), "include", "Inventor", "Win", "SoWin.h")):
                self.MODULES.pop('sowin', None)
                print(red("COINDIR\\include\\Inventor\\Win\\SoWin.h not found. (SoWin bindings won't be built)"))
            print(blue("Checking for QTDIR environment variable..."))
            if os.getenv("QTDIR"):
                print(blue(os.getenv("QTDIR")))
            else:
                self.MODULES.pop('soqt', None)
                print(red("not set. (SoQt bindings won't be built)"))
        else:
            for gui in self.SOGUI:
                if gui not in self.MODULES:
                    continue
                gui_config_cmd = self.MODULES[gui][1]
                if not self.check_cmd_exists(gui_config_cmd):
                    self.MODULES.pop(gui, None)
                else:
                    print(blue("Checking for %s version..." % gui))
                    version = self.do_os_popen("%s --version" % gui_config_cmd)
                    print(blue("%s" % version))

    def get_coin_features(self):
        "set the global variable SWIG_COND_SYMBOLS needed for conditional " + \
        "wrapping"
        if sys.platform == "win32": return
        print(blue("Checking for Coin features..."))
        if not os.system("coin-config --have-feature 3ds_import"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_3DS_IMPORT")
            print(green("3ds import "))

        if not os.system("coin-config --have-feature vrml97"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_VRML97")
            print(green("vrml97 "))

        if not os.system("coin-config --have-feature sound"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_SOUND")
            print(green("sound "))

        if not os.system("coin-config --have-feature superglu"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_SUPERGLUE")
            print(green("superglu "))

        if not os.system("coin-config --have-feature threads"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_THREADS")
            print(green("threads "))

        if not os.system("coin-config --have-feature threadsafe"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_THREADSAFE")
            print(green("threadsafe "))

        print()

    def check_swig_version(self, swig):
        "check for the swig version"
        global SWIG_VERSION
        if not self.check_cmd_exists(swig):
            # on some systems there is only a swig3.0 so check for this and
            # set SWIG to "swig3.0"
            swig = "swig3.0"
            if not self.check_cmd_exists(swig):
                sys.exit(1)
            else:
                self.SWIG = swig
        print(blue("Checking for SWIG version..."))
        p = subprocess.Popen("%s -version" % swig, 
                             shell=True, stdout=subprocess.PIPE)
        version = str(p.stdout.readlines()[1].strip()).split(" ")[2]
        p.stdout.close()
        print(blue("%s" % version))
        SWIG_VERSION = version
        if not version in self.SUPPORTED_SWIG_VERSIONS:
            print(yellow("Warning: Pivy has only been tested with the following " + \
                         "SWIG versions: %s." % " ".join(self.SUPPORTED_SWIG_VERSIONS)))

    def copy_and_swigify_headers(self, includedir, dirname, files):
        """Copy the header files to the local include directories. Add an
        #include line at the beginning for the SWIG interface files..."""
        for file in files:
            if not os.path.isfile(os.path.join(dirname, file)):
                continue

            if file[-2:] == ".i":
                file_i = os.path.join(dirname, file)
                file_h = os.path.join(dirname, file)[:-2] + ".h"

                if (not os.path.exists(file_h) and
                    os.path.exists(os.path.join(includedir, file_h))):
                    shutil.copyfile(os.path.join(includedir, file_h), file_h)
                    sys.stdout.write(' ' + turquoise(file_h))
                    fd = open(file_h, 'r+')
                    contents = fd.readlines()

                    ins_line_nr = -1
                    for line in contents:
                        ins_line_nr += 1
                        if line.find("#include ") != -1:
                            break

                    if ins_line_nr != -1:
                        contents.insert(ins_line_nr, self.pivy_header_include % (file_i))
                        fd.seek(0)
                        fd.writelines(contents)
                    else:
                        print(blue("[") + red("failed") + blue("]"))
                        sys.exit(1)
                    fd.close
            # fixes for SWIG 1.3.21 and upwards
            # (mostly workarounding swig's preprocessor "function like macros"
            # preprocessor bug when no parameters are provided which then results
            # in no constructors being created in the wrapper)
            elif file[-4:] == ".fix":
                sys.stdout.write(' ' + red(os.path.join(dirname, file)[:-4]))
                shutil.copyfile(os.path.join(dirname, file),
                                os.path.join(dirname, file)[:-4])
            # had to introduce this because windows is a piece of crap
            elif sys.platform == "win32" and file[-6:] == ".win32":
                sys.stdout.write(' ' + red(os.path.join(dirname, file)[:-6]))
                shutil.copyfile(os.path.join(dirname, file),
                                os.path.join(dirname, file)[:-6])

    def pivy_configure(self):
        "configure Pivy"
        print(turquoise(self.PIVY_SNAKES))
        print(blue("Platform...%s" % sys.platform))
        self.check_python_version()
        self.check_swig_version(self.SWIG)
        self.check_coin_version()
        self.get_coin_features()
        if self.SOGUI: self.check_gui_bindings()
        
        if 'simvoleon' in self.MODULES and self.check_simvoleon_version():
            if sys.platform == "win32":
                INCLUDE_DIR = os.getenv("SIMVOLEONDIR") + "\\include"
            else:
                INCLUDE_DIR = self.do_os_popen("simvoleon-config --includedir")

            sys.stdout.write(blue("Preparing") + green(" VolumeViz ") + blue("headers:"))
            dir_gen = os.walk("VolumeViz", INCLUDE_DIR)
            for _dir, _, names in dir_gen:
                self.copy_and_swigify_headers(INCLUDE_DIR, _dir, names)
            print(green("."))

        if sys.platform == "win32":
            INCLUDE_DIR = os.path.join(os.getenv("COINDIR"), "include")
        else:
            INCLUDE_DIR = self.do_os_popen("coin-config --includedir")

        sys.stdout.write(blue("Preparing") + green(" Inventor ") + blue("headers:"))
        dir_gen = os.walk("Inventor", INCLUDE_DIR)
        for _dir, _, names in dir_gen:
            self.copy_and_swigify_headers(INCLUDE_DIR, _dir, names)
        print(green("."))

    def swig_generate(self):
        "build all available modules"

        quote = lambda s : '"' + s + '"'
        for module in self.MODULES:
            module_name = self.MODULES[module][0]
            config_cmd = self.MODULES[module][1]
            module_pkg_name = self.MODULES[module][2]
            mod_hack_name = self.MODULES[module][3]
            mod_out_prefix = module_pkg_name.replace('.', os.sep) + module
            
            if sys.platform == "win32":
                INCLUDE_DIR = os.path.join(os.getenv("COINDIR"), "include")
                CPP_FLAGS = "-I" + quote(INCLUDE_DIR) + " " + \
                            "-I" + quote(os.path.join(os.getenv("COINDIR"), "include", "Inventor", "annex")) + \
                            " /DCOIN_DLL /wd4244 /wd4049"
                # aquire highest non-debug Coin library version
                try:
                    LDFLAGS_LIBS = quote(max(glob.glob(os.path.join(os.getenv("COINDIR"), "lib", "coin?.lib")))) + " "
                # with cmake the coin library is named Coin4.lib
                except ValueError:
                    LDFLAGS_LIBS = quote(max(glob.glob(os.path.join(os.getenv("COINDIR"), "lib", "Coin?.lib")))) + " "

                if module == "sowin":
                    CPP_FLAGS += " /DSOWIN_DLL"
                    LDFLAGS_LIBS += quote(os.path.join(os.getenv("COINDIR"), "lib", "sowin1.lib"))
                elif module == "soqt":
                    CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\"  /DSOQT_DLL"
                    if os.path.isdir(os.getenv("QTDIR") + "\\include\Qt\""):
                        CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\Qt\""
                        LDFLAGS_LIBS += os.path.join(os.getenv("COINDIR"), "lib", "soqt1.lib") + " "
                    else:
                        # workaround for conda qt4:
                        CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\qt\Qt\""
                        CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\qt\""
                        LDFLAGS_LIBS += os.path.join(os.getenv("COINDIR"), "lib", "SoQt.lib") + " "
            else:
                INCLUDE_DIR = self.do_os_popen("coin-config --includedir")
                if module_name != 'coin':
                    mod_include_dir = self.do_os_popen("%s --includedir" % config_cmd)
                    if mod_include_dir != INCLUDE_DIR:
                        INCLUDE_DIR += '\" -I\"%s' % mod_include_dir
                CPP_FLAGS = self.do_os_popen("%s --cppflags" % config_cmd) + " -Wno-unused -Wno-maybe-uninitialized"
                LDFLAGS_LIBS = self.do_os_popen("%s --ldflags --libs" % config_cmd)

            if not os.path.isfile(mod_out_prefix + "_wrap.cpp"):
                print(red("\n=== Generating %s_wrap.cpp for %s ===\n" %
                          (mod_out_prefix, module)))
                print(blue(self.SWIG + " " + self.SWIG_SUPPRESS_WARNINGS + " " + self.SWIG_PARAMS %
                           (INCLUDE_DIR,
                            self.CXX_INCS,
                            mod_out_prefix, module)))
                if os.system(self.SWIG + " " + self.SWIG_SUPPRESS_WARNINGS + " " + self.SWIG_PARAMS %
                             (INCLUDE_DIR,
                              self.CXX_INCS,
                              mod_out_prefix, mod_hack_name)):
                    print(red("SWIG did not generate wrappers successfully! ** Aborting **"))
                    sys.exit(1)
            else:
                print(red("=== %s_wrap.cpp for %s already exists! ===" % (mod_out_prefix, module_pkg_name + module)))

            self.ext_modules.append(Extension(module_name, [mod_out_prefix + "_wrap.cpp"],
                                              extra_compile_args=(self.CXX_INCS + CPP_FLAGS).split(),
                                              extra_link_args=(self.CXX_LIBS + LDFLAGS_LIBS).split()))

    def run(self):
        "the entry point for the distutils build class"
        if sys.platform == "win32" and not os.getenv("COINDIR"):
            print("Please set the COINDIR environment variable to your Coin root directory! ** Aborting **")
            sys.exit(1)

        self.pivy_configure()
        self.swig_generate()

        for cmd_name in self.get_sub_commands():
            self.run_command(cmd_name)

class pivy_clean(clean):
    pivy_path = 'pivy' + os.sep
    gui_path = 'pivy' + os.sep + 'gui' + os.sep
    REMOVE_FILES = (pivy_path + '__init__.pyc', gui_path + '__init__.pyc',
                    pivy_path + 'coin_wrap.cpp', pivy_path + 'coin2_wrap.cpp',  pivy_path + 'coin.py',  pivy_path + 'coin.pyc',
                    pivy_path + 'simvoleon_wrap.cpp',  pivy_path + 'simvoleon.py',  pivy_path + 'simvoleon.pyc',
                    gui_path + 'soqt_wrap.cpp',  gui_path + 'soqt.py',  gui_path + 'soqt.pyc',
                    gui_path + 'sogtk_wrap.cpp', gui_path + 'sogtk.py', gui_path + 'sogtk.py',
                    gui_path + 'soxt_wrap.cpp',  gui_path + 'soxt.py',  gui_path + 'soxt.pyc',
                    gui_path + 'sowin_wrap.cpp', gui_path + 'sowin.py', gui_path + 'sowin.pyc',
                    pivy_path + 'sogui.pyc')

    def remove_headers(self, arg, dirname, files):
        "remove the coin headers from the pivy Inventor directory"
        for file in files:
            if not os.path.isfile(os.path.join(dirname, file)) or file[-2:] != ".h":
                continue
            sys.stdout.write(' ' + turquoise(os.path.join(dirname, file)))
            os.remove(os.path.join(dirname, file))

    def run(self):
        "the entry point for the distutils clean class"
        sys.stdout.write(blue("Cleaning headers:"))
        dir_gen = os.walk("Inventor")
        for _dir, _, names in dir_gen:
            self.remove_headers(None, _dir, names)

        dir_gen = os.walk("VolumeViz")
        for _dir, _, names in dir_gen:
            self.remove_headers(None, _dir, names)

        # remove the SWIG generated wrappers
        for wrapper_file in self.REMOVE_FILES:
            if os.path.isfile(wrapper_file):
                sys.stdout.write(' ' + turquoise(wrapper_file))
                os.remove(wrapper_file)
        print(green("."))

        clean.run(self)

for i in reversed(list(range(len(sys.argv)))):
    if sys.argv[i][:10] == "--without-":
        pivy_build.MODULES.pop(sys.argv[i][10:], None)
        del sys.argv[i]

setup(name = "Pivy",
      version = PIVY_VERSION,
      description = "A Python binding for Coin",
      long_description = __doc__,
      author = "Tamer Fahmy",
      author_email = "tamer@sim.no",
      download_url="https://github.com/coin3d/pivy/releases",
      url = "http://pivy.coin3d.org/",
      cmdclass = {'build'   : pivy_build,
                  'clean'   : pivy_clean},
      ext_package = 'pivy',
      ext_modules = pivy_build.ext_modules,
      py_modules  = pivy_build.py_modules,
      packages = ['pivy', 'pivy.gui'],
      classifiers = [_f for _f in PIVY_CLASSIFIERS.split("\n") if _f],
      license = "BSD License",
      platforms = ['Any']
      )



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 5, example 3.
#
# This example creates a TriangleStripSet. It creates
# a pennant-shaped flag.
#

import sys

from pivy.coin import *
from pivy.sogui import *

##############################################################
## CODE FOR The Inventor Mentor STARTS HERE

#
# Positions of all of the vertices:
#
vertexPositions = (
   (  0,   12,    0 ), (   0,   15,    0),
   (2.1, 12.1,  -.2 ), ( 2.1, 14.6,  -.2),
   (  4, 12.5,  -.7 ), (   4, 14.5,  -.7),
   (4.5, 12.6,  -.8 ), ( 4.5, 14.4,  -.8),
   (  5, 12.7,   -1 ), (   5, 14.4,   -1),
   (4.5, 12.8, -1.4 ), ( 4.5, 14.6, -1.4),
   (  4, 12.9, -1.6 ), (   4, 14.8, -1.6),
   (3.3, 12.9, -1.8 ), ( 3.3, 14.9, -1.8),
   (  3,   13, -2.0 ), (   3, 14.9, -2.0), 
   (3.3, 13.1, -2.2 ), ( 3.3, 15.0, -2.2),
   (  4, 13.2, -2.5 ), (   4, 15.0, -2.5),
   (  6, 13.5, -2.2 ), (   6, 14.8, -2.2),
   (  8, 13.4,   -2 ), (   8, 14.6,   -2),
   ( 10, 13.7, -1.8 ), (  10, 14.4, -1.8),
   ( 12,   14, -1.3 ), (  12, 14.5, -1.3),
   ( 15, 14.9, -1.2 ), (  15,   15, -1.2),

   (-.5, 15,   0 ), ( -.5, 0,   0),   # the flagpole
   (  0, 15,  .5 ), (   0, 0,  .5),
   (  0, 15, -.5 ), (   0, 0, -.5),
   (-.5, 15,   0 ), ( -.5, 0,   0)
)


# Number of vertices in each strip.
numVertices = (
   32, # flag
   8   # pole
)
 
# Colors for the 12 faces
colors = (
   ( .5, .5,  1 ), # purple flag
   ( .4, .4, .4 ), # grey flagpole
)

# set this variable to 0 if you want to use the other method
IV_STRICT = 1

# Routine to create a scene graph representing a pennant.
def makePennant():
    result = SoSeparator()

    # A shape hints tells the ordering of polygons. 
    # This insures double sided lighting.
    myHints = SoShapeHints()
    myHints.vertexOrdering = SoShapeHints.COUNTERCLOCKWISE
    result.addChild(myHints)

    if IV_STRICT:
        # This is the preferred code for Inventor 2.1 

        # Using the new SoVertexProperty node is more efficient
        myVertexProperty = SoVertexProperty()

        # Define colors for the strips
        for i in range(2):
            myVertexProperty.orderedRGBA.set1Value(i, SbColor(colors[i]).getPackedValue())
            myVertexProperty.materialBinding = SoMaterialBinding.PER_PART

        # Define coordinates for vertices
        myVertexProperty.vertex.setValues(0, 40, vertexPositions)

        # Define the TriangleStripSet, made of two strips.
        myStrips = SoTriangleStripSet()
        myStrips.numVertices.setValues(0, 2, numVertices)
 
        myStrips.vertexProperty = myVertexProperty
        result.addChild(myStrips)

    else:
        # Define colors for the strips
        myMaterials = SoMaterial()
        myMaterials.diffuseColor.setValues(0, 2, colors)
        result.addChild(myMaterials)
        myMaterialBinding = SoMaterialBinding()
        myMaterialBinding.value = SoMaterialBinding.PER_PART
        result.addChild(myMaterialBinding)

        # Define coordinates for vertices
        myCoords = SoCoordinate3()
        myCoords.point.setValues(0, 40, vertexPositions)
        result.addChild(myCoords)

        # Define the TriangleStripSet, made of two strips.
        myStrips = SoTriangleStripSet()
        myStrips.numVertices.setValues(0, 2, numVertices)
        result.addChild(myStrips)

    return result

## CODE FOR The Inventor Mentor ENDS HERE
##############################################################

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = makePennant()

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Triangle Strip Set: Pennant")
    myViewer.show()
    myViewer.viewAll()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

##
# Demonstrates embedding of an SoQtExaminerViewer within a simple widget
# hierarchy.

import sys
from random import random

from pivy.coin import *
from pivy.gui.soqt import *

from PySide2.QtGui import *
from PySide2.QtCore import *

class EmbeddedWindow(QMainWindow):
    def __init__(self, *args):
        super(EmbeddedWindow, self).__init__(*args)

        # dummy widget needed for the PyQt stuff
        self.centralWidget = QWidget(self)
        self.mainLayout = QVBoxLayout(self.centralWidget)
        self.examiner = QWidget(self.centralWidget)
        self.mainLayout.addWidget(self.examiner)
        self.hLayout = QHBoxLayout()
        self.groupBox = QGroupBox("Choose axis", self.centralWidget)
        self.hboxlayout1 = QHBoxLayout(self.groupBox)
        self.buttonGroup = QButtonGroup(self.groupBox)
        self.radio_x = QRadioButton("&X", self.groupBox)
        self.radio_y = QRadioButton("&Y", self.groupBox)
        self.radio_z = QRadioButton("&Z", self.groupBox)
        self.buttonGroup.addButton(self.radio_x, 0)
        self.buttonGroup.addButton(self.radio_y, 1)
        self.buttonGroup.addButton(self.radio_z, 2)
        self.hboxlayout1.addWidget(self.radio_x)
        self.hboxlayout1.addWidget(self.radio_y)
        self.hboxlayout1.addWidget(self.radio_z)
        self.hLayout.addWidget(self.groupBox)

        self.controlLayout = QVBoxLayout()
        self.checkbox = QCheckBox("Enable &rotation", self.centralWidget)
        self.checkbox.setDown(False)
        self.button = QPushButton("&Change cone color", self.centralWidget)
        self.controlLayout.addWidget(self.checkbox)
        self.controlLayout.addWidget(self.button)

        self.hLayout.addLayout(self.controlLayout)
        self.mainLayout.addLayout(self.hLayout)

        self.setCentralWidget(self.centralWidget)

        self.radio_x.setChecked(True)

        # construct a simple scenegraph
        root = SoSeparator()
        self.rotxyz = SoRotationXYZ()
        self.gate = SoGate(SoMFFloat.getClassTypeId())
        self.elapsedTime = SoElapsedTime()
        self.gate.enable = False
        self.gate.input.connectFrom(self.elapsedTime.timeOut)
        self.rotxyz.angle.connectFrom(self.gate.output)
        self.material = SoMaterial()
        self.material.diffuseColor = (0.0, 1.0, 1.0)
        self.cone = SoCone()
        root.addChild(self.rotxyz)
        root.addChild(self.material)
        root.addChild(self.cone)

        # N.B.: keep in mind that it is important to keep the examiner
        # viewer as an instance variable by prefixing it with 'self.'
        # otherwise it will fall out of scope and gets deallocated ->
        # no redraws and crashes. 20050727 tamer.

        # add the examinerviewer
        self.exam = SoQtExaminerViewer(self.examiner)
        self.exam.setSceneGraph(root)
        self.exam.setTitle("Embedded viewer")
        self.exam.show()

        self.connect(self.buttonGroup, SIGNAL("buttonClicked(int)"), self.change_axis)
        self.connect(self.button, SIGNAL("clicked()"), self.change_color)
        self.connect(self.checkbox, SIGNAL("clicked()"), self.rotate)

    def change_axis(self, axis):
        self.rotxyz.axis = axis

    def change_color(self):
        self.material.diffuseColor = (random(), random(), random())

    def rotate(self):
        self.gate.enable = not self.gate.enable.getValue()

def main():
    qApp = QApplication([])
    # initialize Qt and SoQt
    SoQt.init(None)

    # set up scrollview window
    vp = EmbeddedWindow()
    # map window
    vp.resize(640, 480)

    # set termination condition
    QObject.connect(qApp, SIGNAL("lastWindowClosed()"), qApp, SLOT("quit()"))

    # start event loop
    SoQt.mainLoop()

if __name__ == '__main__':
    main()



from __future__ import print_function
import sys
from pivy import coin
from pivy.gui import soqt
from PySide2.QtCore import QEvent


def foo(a, event):
    if event.type() == QEvent.MouseButtonPress:
        print(event)  # event is not a QMouseEvent like it was with pyqt4
        print(event.button())


appWindow = soqt.SoQt.init(sys.argv[0])
root = coin.SoSeparator()
myRenderArea = soqt.SoQtRenderArea(appWindow)
myRenderArea.setSceneGraph(root)
myRenderArea.setTitle("My Event Handler")
myRenderArea.setEventCallback(foo, myRenderArea)

myRenderArea.show()
soqt.SoQt.show(appWindow)
soqt.SoQt.mainLoop()



#!/usr/bin/env pythonw

from AppKit import *
from Foundation import *
from PyObjCTools import NibClassBuilder, AppHelper
import sys, objc

objc.loadBundle('Sc21', globals(),
                bundle_path='/Library/Frameworks/Sc21.framework')

NibClassBuilder.extractClasses('MainMenu.nib')
            
class AppController (NibClassBuilder.AutoBaseClass):

    def awakeFromNib(self):
        self.filenametext.setStringValue_(u"None")

    def open_(self, sender):
        panel = NSOpenPanel.openPanel()
        panel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo_(None, None, [u'wrl'], NSApp().mainWindow(), self, "openPanelDidEnd:returnCode:contextInfo:", 0)
    def openPanelDidEnd_returnCode_contextInfo_(self, panel, code, ct):
        if code == NSOKButton:
            sg = self.coincontroller.sceneGraph()
            sg.readFromFile_(panel.filename())
            sg.viewAll()
            self.filenametext.setStringValue_(panel.filename())

    openPanelDidEnd_returnCode_contextInfo_ =  AppHelper.endSheetMethod(
        openPanelDidEnd_returnCode_contextInfo_)

if __name__ == "__main__":
    sys.exit(NSApplicationMain(sys.argv))



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 5, example 4.
#
# This example creates the St. Louis Arch using a QuadMesh.
#

import sys

from pivy.coin import *
from pivy.sogui import *

##############################################################
## CODE FOR The Inventor Mentor STARTS HERE

# Positions of all of the vertices:
vertexPositions = (
   # 1st row
   (-13.0,  0.0, 1.5), (-10.3, 13.7, 1.2), ( -7.6, 21.7, 1.0), 
   ( -5.0, 26.1, 0.8), ( -2.3, 28.2, 0.6), ( -0.3, 28.8, 0.5),
   (  0.3, 28.8, 0.5), (  2.3, 28.2, 0.6), (  5.0, 26.1, 0.8), 
   (  7.6, 21.7, 1.0), ( 10.3, 13.7, 1.2), ( 13.0,  0.0, 1.5),
   # 2nd row
   (-10.0,  0.0, 1.5), ( -7.9, 13.2, 1.2), ( -5.8, 20.8, 1.0), 
   ( -3.8, 25.0, 0.8), ( -1.7, 27.1, 0.6), ( -0.2, 27.6, 0.5),
   (  0.2, 27.6, 0.5), (  1.7, 27.1, 0.6), (  3.8, 25.0, 0.8), 
   (  5.8, 20.8, 1.0), (  7.9, 13.2, 1.2), ( 10.0,  0.0, 1.5),
   # 3rd row
   (-10.0,  0.0,-1.5), ( -7.9, 13.2,-1.2), ( -5.8, 20.8,-1.0), 
   ( -3.8, 25.0,-0.8), ( -1.7, 27.1,-0.6), ( -0.2, 27.6,-0.5),
   (  0.2, 27.6,-0.5), (  1.7, 27.1,-0.6), (  3.8, 25.0,-0.8), 
   (  5.8, 20.8,-1.0), (  7.9, 13.2,-1.2), ( 10.0,  0.0,-1.5),
   # 4th row 
   (-13.0,  0.0,-1.5), (-10.3, 13.7,-1.2), ( -7.6, 21.7,-1.0), 
   ( -5.0, 26.1,-0.8), ( -2.3, 28.2,-0.6), ( -0.3, 28.8,-0.5),
   (  0.3, 28.8,-0.5), (  2.3, 28.2,-0.6), (  5.0, 26.1,-0.8), 
   (  7.6, 21.7,-1.0), ( 10.3, 13.7,-1.2), ( 13.0,  0.0,-1.5),
   # 5th row
   (-13.0,  0.0, 1.5), (-10.3, 13.7, 1.2), ( -7.6, 21.7, 1.0), 
   ( -5.0, 26.1, 0.8), ( -2.3, 28.2, 0.6), ( -0.3, 28.8, 0.5),
   (  0.3, 28.8, 0.5), (  2.3, 28.2, 0.6), (  5.0, 26.1, 0.8), 
   (  7.6, 21.7, 1.0), ( 10.3, 13.7, 1.2), ( 13.0,  0.0, 1.5)
)

# set this variable to 0 if you want to use the other method
IV_STRICT = 0

# Routine to create a scene graph representing an arch.
def makeArch():
   result = SoSeparator()

   if IV_STRICT:
       # This is the preferred code for Inventor 2.1 

       # Using the new SoVertexProperty node is more efficient
       myVertexProperty = SoVertexProperty()

       # Define the material
       myVertexProperty.orderedRGBA = SbColor(.78, .57, .11).getPackedValue()

       # Define coordinates for vertices
       myVertexProperty.vertex.setValues(0, 60, vertexPositions)

       # Define the QuadMesh.
       myQuadMesh = SoQuadMesh()
       myQuadMesh.verticesPerRow(12)

       myQuadMesh.verticesPerColumn(5)

       myQuadMesh.vertexProperty = myVertexProperty
       result.addChild(myQuadMesh)

   else:
       # Define the material
       myMaterial = SoMaterial()
       myMaterial.diffuseColor = (.78, .57, .11)
       result.addChild(myMaterial)

       # Define coordinates for vertices
       myCoords = SoCoordinate3()
       myCoords.point.setValues(0, 60, vertexPositions)
       result.addChild(myCoords)

       # Define the QuadMesh.
       myQuadMesh = SoQuadMesh()
       myQuadMesh.verticesPerRow = 12

       myQuadMesh.verticesPerColumn = 5
       result.addChild(myQuadMesh)

   return result

## CODE FOR The Inventor Mentor ENDS HERE
##############################################################

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = makeArch()

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Quad Mesh: Arch")
    myViewer.show()
    myViewer.viewAll()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# Pivy Coin unit test suite
#
# For detailed info on its usage and on how to write additional test cases
# read:
#   - http://pyunit.sourceforge.net/pyunit.html
#   - http://diveintopython.org/unit_testing/
#
# Invoke this script with '--help' for usage information.
#

from pivy.coin import *
import unittest
import math

class Autocasting(unittest.TestCase):
    def testFieldAutocast(self):
        """check autocasting for SoSFBool created through createInstance()"""
        t = SoType.fromName(SbName("SoSFBool"))
        self.failUnless(isinstance(t.createInstance(), SoSFBool),
                        'not autocasted to an SoSFBool instance')

    def testNodeAutocast(self):
        """check autocasting for SoCone created through createInstance()"""
        t = SoType.fromName(SbName("SoCone"))
        self.failUnless(isinstance(t.createInstance(), SoCone),
                        'not autocasted to an SoCone instance')

    def testNodeListAutocast(self):
        """check autocasting for elements in NodeList"""
        nl = SoNodeList()
        nl.append(SoCube())
        self.failUnless(isinstance(nl.get(0), SoCube),
                        'not autocasted to an SoCube instance')

    def testTypeAutocast(self):
        """check if type objects return casted instances"""
        tCube = SoType.fromName(SbName("SoCube"))
        cube = tCube.createInstance()
        self.failUnless(isinstance(cube, SoCube),
                        'SoType.createInstance not casted to SoCube')
        tSFBool = SoType.fromName(SbName("SoSFBool"))
        field = tSFBool.createInstance()
        self.failUnless(isinstance(field, SoSFBool),
                        'SoType.createInstance not casted to SoSFBool')
        tPath = SoType.fromName(SbName("SoPath"))
        path = tPath.createInstance()
        self.failUnless(isinstance(path, SoPath),
                        'SoType.createInstance not casted to SoPath') 

    def testFieldContainerAutocast(self):
        """check if a returned FieldContainer is casted"""
        m = SoMaterial()
        self.failUnless(isinstance(m.diffuseColor.getContainer(), SoMaterial),
                        'SoField.getContainer is not casted correctly')
                        
class FieldSetValue(unittest.TestCase):
    """checks various setValue(s) calls for fields"""
    def testSFBool(self):
        """check setValue for SoSFBool"""
        t = SoSFBool()
        s = SoSFBool()
        t.setValue(True)
        s.setValue(t)
        self.failUnless(True == t.getValue() == s.getValue(), 
                        'setValue on SoSFBool failed')

    def testSFColor(self):
        """check setValue for SoSFColor"""
        t = SoSFColor()
        s = SoSFColor()
        t.setValue(0,1,1)
        self.failUnless(t.getValue() == SbColor(0,1,1), 
                        'setValue 3 floats on SoSFColor failed')
        t.setValue([0,1,0])
        self.failUnless(t.getValue() == SbColor(0,1,0), 
                        'setValue float sequence on SoSFColor failed')
        t.setValue(SbColor(1,0,0))
        self.failUnless(t.getValue() == SbColor(1,0,0), 
                        'setValue SbColor on SoSFColor failed')
        t.setValue(SbVec3f(1,0,1))
        self.failUnless(t.getValue() == SbColor(1,0,1), 
                        'setValue SbVec3f on SoSFColor failed') 
        s.setValue(t)
        self.failUnless(t.getValue() == s.getValue(),
                        'setValue othe SoSFcolor on SoSFColor failed')
        
    def testSFEngine(self):
        """check setValue for SoSFEngine"""
        t = SoSFEngine()
        s = SoSFEngine()
        c = SoCounter()
        t.setValue(c)
        self.failUnless(t.getValue() == c, 
                        'setValue engine on SoSFEngine failed')
        s.setValue(t)
        self.failUnless(t.getValue() == s.getValue(), 
                        'setValue other SoSFEngine on SoSFEngine failed')

    def testSFString(self):
        """check setValue for SoSFString"""
        t = SoSFString()
        s = SoSFString()
        c = SbString('bla')
        t.setValue(c)
        self.failUnless(t.getValue() == c, 
                        'setValue SbString on SoSFString failed')
        t.setValue('hello')
        self.failUnless(t.getValue() == 'hello', 
                        'setValue string on SoSFString failed')        
        s.setValue(t)
        self.failUnless(t.getValue() == s.getValue(), 
                        'setValue other SoSFString on SoSFString failed')
        
    def testSFInt32(self):
        """check setValue for SoSFInt32"""
        t = SoSFInt32()
        s = SoSFInt32()
        t.setValue(10)
        s.setValue(t)
        self.failUnless(10 == t.getValue() == s.getValue(), 
                        'setValue on SoSFInt32 failed')

    def testSFFloat(self):
        """check setValue for SoSFFloat"""
        t = SoSFFloat()
        s = SoSFFloat()
        t.setValue(10.5)
        s.setValue(t)
        self.failUnless(10.5 == t.getValue() == s.getValue(), 
                        'setValue on SoSFFloat failed')

    def testSFShort(self):
        """check setValue for SoSFShort"""
        t = SoSFShort()
        s = SoSFShort()
        t.setValue(10)
        s.setValue(t)
        self.failUnless(10 == t.getValue() == s.getValue(), 
                        'setValue on SoSFShort failed')

    def testSFMatrix(self):
        """check setValue for SoSFMatrix"""
        t = SoSFMatrix()
        s = SoSFMatrix()
        # m = SbMatrix([[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0]])
        m2 = SbMatrix([[1,1,1,1],[1,1,1,1],[1,1,1,1],[1,1,1,1]])
        t.setValue(m2)
        s.setValue(t)
        self.failUnless(m2 == t.getValue() == s.getValue(), 
                        'setValue on SoSFMatrix failed')                       

    def testSFName(self):
        """check setValue for SoSFName"""
        t = SoSFName()
        s = SoSFName()
        c = SbName('bla')
        t.setValue(c)
        self.failUnless(t.getValue() == c, 
                        'setValue SbName on SoSFName failed')
        t.setValue('hello')
        self.failUnless(t.getValue() == 'hello', 
                        'setValue string on SoSFName failed')        
        s.setValue(t)
        self.failUnless(t.getValue() == s.getValue(), 
                        'setValue other SoSFName on SoSFName failed')

    def testSFNode(self):
        """check setValue for SoSFNode"""
        t = SoSFNode()
        s = SoSFNode()
        c = SoCone()
        t.setValue(c)
        s.setValue(t)
        self.failUnless(c == t.getValue() == s.getValue(), 
                        'setValue on SoSFNode failed')
        self.failUnless(isinstance(t.getValue(), SoCone), 
                        'autocast on SoSFNode.getValue failed')
        
    def testSFRotation(self):
        """check setValue for SoSFRotation"""
        t = SoSFRotation()
        s = SoSFRotation()
        m = SbRotation(1,0,0,0)
        t.setValue(m)
        s.setValue(t)
        self.failUnless(m == t.getValue() == s.getValue(), 
                        'setValue on SoSFRotation failed') 
        t.setValue(0,1,0,0)
        self.failUnless(SbRotation(0,1,0,0) == t.getValue(),
                        'setValue on SoSFRotation from 4 values failed')
        t.setValue([0,0,1,0])
        self.failUnless(SbRotation(0,0,1,0) == t.getValue(),
                        'setValue on SoSFRotation from 4 values failed')

    def testSoRotation(self):
        r = SoRotation()
        rot = r.rotation.getValue()
        rot *= SbRotation(SbVec3f(0,0,1), M_PI/90.0)
        r.rotation.setValue(rot)

    def testSFVec3f(self):
        """check setValue for SoSFVec3f"""
        t = SoSFVec3f()
        s = SoSFVec3f()
        t.setValue(0,1,1)
        self.failUnless(t.getValue() == SbVec3f(0,1,1), 
                        'setValue 3 floats on SoSFVec3f failed')
        t.setValue([0,1,0])
        self.failUnless(t.getValue() == SbVec3f(0,1,0), 
                        'setValue float sequence on SoSFVec3f failed')
        t.setValue(SbVec3f(1,0,0))
        self.failUnless(t.getValue() == SbVec3f(1,0,0), 
                        'setValue SbVec3f on SoSFVec3f failed')
        s.setValue(t)
        self.failUnless(t.getValue() == s.getValue(),
                        'setValue other SoSFVec3f on SoSFVec3f failed')


    def testSFVec2f(self):
        """check setValue for SoSFVec2f"""
        t = SoSFVec2f()
        s = SoSFVec2f()
        t.setValue(0,1)
        self.failUnless(t.getValue() == SbVec2f(0,1), 
                        'setValue 2 floats on SoSFVec2f failed')
        t.setValue([1,0])
        self.failUnless(t.getValue() == SbVec2f(1,0), 
                        'setValue float sequence on SoSFVec2f failed')
        t.setValue(SbVec2f(1,1))
        self.failUnless(t.getValue() == SbVec2f(1,1), 
                        'setValue SbVec2f on SoSFVec2f failed')
        s.setValue(t)
        self.failUnless(t.getValue() == s.getValue(),
                        'setValue other SoSFVec2f on SoSFVec2f failed')

    def testSFVec4f(self):
        """check setValue for SoSFVec4f"""
        t = SoSFVec4f()
        s = SoSFVec4f()
        t.setValue(0,1,1,0)
        self.failUnless(t.getValue() == SbVec4f(0,1,1,0), 
                        'setValue 4 floats on SoSFVec4f failed')
        t.setValue([0,1,0,1])
        self.failUnless(t.getValue() == SbVec4f(0,1,0,1), 
                        'setValue float sequence on SoSFVec4f failed')
        t.setValue(SbVec4f(1,0,0,1))
        self.failUnless(t.getValue() == SbVec4f(1,0,0,1), 
                        'setValue SbVec4f on SoSFVec4f failed')
        s.setValue(t)
        self.failUnless(t.getValue() == s.getValue(),
                        'setValue other SoSFVec4f on SoSFVec4f failed')

    def testSFImage(self):
        """check setValue for SoSFImage"""
        t = SoSFImage()
        s = SoSFImage()        
        t.setValue(SbVec2s(2,2), 1, "abcd")
        s.setValue(t)
        self.failUnless(("abcd", SbVec2s(2,2), 1) == t.getValue() == s.getValue(), 
                        'setValue on SoSFImage failed')

    def testSFImage3(self):
        """check setValue for SoSFImage3"""
        t = SoSFImage3()
        s = SoSFImage3()        
        t.setValue(SbVec3s(2,2,2), 1, "abcdefgh")
        s.setValue(t)
        self.failUnless(("abcdefgh", SbVec3s(2,2,2), 1) == t.getValue() == s.getValue(), 
                        'setValue on SoSFImage3 failed')

    def testSFPath(self):
        """check setValue for SoSFPath"""
        t = SoSFPath()
        s = SoSFPath()
        c = SoPath()
        c.ref()
        t.setValue(c)
        s.setValue(t)
        self.failUnless(c == t.getValue() == s.getValue(), 
                        'setValue on SoSFPath failed')
        self.failUnless(isinstance(t.getValue(), SoPath), 
                        'autocast on SoSFPath.getValue failed')

    def testSFPlane(self):
        """check setValue for SoSFPlane"""
        t = SoSFPlane()
        s = SoSFPlane()
        c = SbPlane()
        t.setValue(c)
        s.setValue(t)
        self.failUnless(c == t.getValue() == s.getValue(), 
                        'setValue on SoSFPlane failed')

    def testSFUInt32(self):
        """check setValue for SoSFUInt32"""
        t = SoSFUInt32()
        s = SoSFUInt32()
        t.setValue(10)
        s.setValue(t)
        self.failUnless(10 == t.getValue() == s.getValue(), 
                        'setValue on SoSFUInt32 failed')

    def testSFUShort(self):
        """check setValue for SoSFUShort"""
        t = SoSFUShort()
        s = SoSFUShort()
        t.setValue(10)
        s.setValue(t)
        self.failUnless(10 == t.getValue() == s.getValue(), 
                        'setValue on SoSFUShort failed')

    def testSFTime(self):
        """check setValue for SoSFTime"""
        t = SoSFTime()
        s = SoSFTime()
        t.setValue(150.5)
        s.setValue(t)
        self.failUnless(150.5 == t.getValue() == s.getValue(), 
                        'setValue on SoSFTime failed')

    def testMFBool(self):
        """check setValue(s) for SoMFBool"""
        t = SoMFBool()
        s = SoMFBool()
        t.setValues([0,1])
        self.failUnless(t.getValues() == [0,1],
                        'setValues with sequence on SoMFBool failed')
        t.setValues(2,[0,1])
        self.failUnless(t.getValues() == [0,1,0,1],
                        'setValues with start and sequence on SoMFBool failed')
        t.setValues(0,1,[1,0])
        self.failUnless(t.getValues() == [1,1,0,1],
                        'setValues with start, length and sequence on SoMFBool failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFBool on SoMFBool failed')
        t.setValue(0)
        self.failUnless(t.getValues() == [0],
                        'setValue with bool on SoMFBool failed')

    def testMFColor(self):
        """check setValue(s) for SoMFColor"""
        t = SoMFColor()
        s = SoMFColor()
        t.setValues([[0,0,1]])
        t.setValues(1,[[0,0,1]])
        t.setValues(2,1,[[0,0,1],[1,0,1]])
        self.failUnless(t.getValues() == [SbColor(0,0,1),SbColor(0,0,1),SbColor(0,0,1)],
                        'setValues with sequence of float triples on SoMFColor failed')                    
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with another SoMFColor failed' )
        t.setValues([SbColor(0,0,1)])
        t.setValues(1,[SbColor(0,0,1)])
        t.setValues(2,1,[SbColor(0,0,1), SbColor(1,0,1)])
        self.failUnless(t.getValues() == [SbColor(0,0,1),SbColor(0,0,1),SbColor(0,0,1)],
                        'setValues with sequence of SbColors on SoMFColor failed')                    
        t.setValues([])
        self.failUnless(t.getValues() == [SbColor(0,0,1),SbColor(0,0,1),SbColor(0,0,1)],
                        'setValue with empty sequence of SbColors failed' )

    def testMFEngine(self):
        """check setValue(s) for SoMFEngine"""
        t = SoMFEngine()
        s = SoMFEngine()
        c = SoCounter()
        c.ref()
        c2 = SoCounter()
        c2.ref()
        t.setValue(c)
        self.failUnless(t.getValues() == [c], 
                        'setValue engine on SoMFEngine failed')
        t.setValues([c,c2])
        self.failUnless(t.getValues() == [c,c2], 
                        'setValues on SoMFEngine failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFEngine on SoMFEngine failed')
        t.setValues(2,1,[c,c2])
        self.failUnless(t.getValues() == [None,None,c],
                        'setValues with start, length and sequence on SoMFEngine failed')

    def testMFEnum(self):
        """check setValue(s) for SoMFEnum"""
        t = SoMFEnum()
        s = SoMFEnum()
        t.setValues([0,2])
        self.failUnless(t.getValues() == [0,2],
                        'setValues with sequence on SoMFEnum failed')
        t.setValues(2,[0,1])
        self.failUnless(t.getValues() == [0,2,0,1],
                        'setValues with start and sequence on SoMFEnum failed')
        t.setValues(0,1,[-1,0])
        self.failUnless(t.getValues() == [-1,2,0,1],
                        'setValues with start, length and sequence on SoMFEnum failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFEnum on SoMFEnum failed')
        t.setValue(0)
        self.failUnless(t.getValues() == [0],
                        'setValue with single int on SoMFEnum failed')
        t.setValues([])
        self.failUnless(t.getValues() == [0],
                        'setValues with empty list on SoMFEnum failed')

    def testMFString(self):
        """check setValue(s) for SoMFString"""
        t = SoMFString()
        s = SoMFString()
        t.setValues(['1','2'])
        t.setValues(1,['3'])
        t.setValues(2,1,['4','5'])
        self.failUnless(t.getValues() == ['1','3','4'],
                        'setValues with sequence of strings on SoMFString failed')                    
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with another SoMFString failed' )
        t.setValues([SbString('1')])
        t.setValues(1,[SbString('2')])
        t.setValues(2,1,[SbString('3'), SbString('4')])
        self.failUnless(t.getValues() == ['1','2','3'],
                        'setValues with sequence of SbStrings on SoMFString failed')                    


    def testMFInt32(self):
        """check setValue(s) for SoMFInt32"""
        t = SoMFInt32()
        s = SoMFInt32()
        t.setValues([0,2])
        self.failUnless(t.getValues() == [0,2],
                        'setValues with sequence on SoMFInt32 failed')
        t.setValues(2,[0,1])
        self.failUnless(t.getValues() == [0,2,0,1],
                        'setValues with start and sequence on SoMFInt32 failed')
        t.setValues(0,1,[-1,0])
        self.failUnless(t.getValues() == [-1,2,0,1],
                        'setValues with start, length and sequence on SoMFInt32 failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFInt32 on SoMFInt32 failed')
        t.setValue(0)
        self.failUnless(t.getValues() == [0],
                        'setValue with single int on SoMFInt32 failed')
        t.setValues([])
        self.failUnless(t.getValues() == [0],
                        'setValues with empty list on SoMFInt32 failed')

    def testMFFloat(self):
        """check setValue(s) for SoMFFloat"""
        t = SoMFFloat()
        s = SoMFFloat()
        t.setValues([0.5,2])
        self.failUnless(t.getValues() == [0.5,2],
                        'setValues with sequence on SoMFFloat failed')
        t.setValues(2,[0,1])
        self.failUnless(t.getValues() == [0.5,2,0,1],
                        'setValues with start and sequence on SoMFFloat failed')
        t.setValues(0,1,[1.5,0])
        self.failUnless(t.getValues() == [1.5,2,0,1],
                        'setValues with start, length and sequence on SoMFFloat failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFFloat on SoMFFloat failed')
        t.setValue(-0.5)
        self.failUnless(t.getValues() == [-0.5],
                        'setValue with single int on SoMFFloat failed')
        t.setValues([])
        self.failUnless(t.getValues() == [-0.5],
                        'setValues with empty list on SoMFFloat failed')     

    def testMFShort(self):
        """check setValue(s) for SoMFShort"""
        t = SoMFShort()
        s = SoMFShort()
        t.setValues([0,2])
        self.failUnless(t.getValues() == [0,2],
                        'setValues with sequence on SoMFShort failed')
        t.setValues(2,[0,1])
        self.failUnless(t.getValues() == [0,2,0,1],
                        'setValues with start and sequence on SoMFShort failed')
        t.setValues(0,1,[-1,0])
        self.failUnless(t.getValues() == [-1,2,0,1],
                        'setValues with start, length and sequence on SoMFShort failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFShort on SoMFShort failed')
        t.setValue(0)
        self.failUnless(t.getValues() == [0],
                        'setValue with single int on SoMFShort failed')
        t.setValues([])
        self.failUnless(t.getValues() == [0],
                        'setValues with empty list on SoMFShort failed')
        
    def testMFMatrix(self):
        """check setValue(s) for SoMFMatrix"""
        t = SoMFMatrix()
        s = SoMFMatrix()
        m = SbMatrix([[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0]])
        m2 = SbMatrix([[1,1,1,1],[1,1,1,1],[1,1,1,1],[1,1,1,1]])
        t.setValues([m])
        self.failUnless(t.getValues() == [m],
                        'setValues with sequence on SoMFMatrix failed')
        t.setValues(2,[m2,m2])
        self.failUnless(t.getValues()[2:4] == [m2,m2],
                        'setValues with start and sequence on SoMFMatrix failed')
        t.setValues(1,1,[m2,m])
        self.failUnless(t.getValues() == [m,m2,m2,m2],
                        'setValues with start, length and sequence on SoMFMatrix failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFMatrix on SoMFMatrix failed')
        t.setValue(m2)
        self.failUnless(t.getValues() == [m2],
                        'setValue with single int on SoMFMatrix failed')
        t.setValues([])
        self.failUnless(t.getValues() == [m2],
                        'setValues with empty list on SoMFMatrix failed')                     

    def testMFName(self):
        """check setValue(s) for SoMFName"""
        t = SoMFName()
        s = SoMFName()
        t.setValues(['1','2'])
        t.setValues(1,['3'])
        t.setValues(2,1,['4','5'])
        self.failUnless(t.getValues() == ['1','3','4'],
                        'setValues with sequence of Names on SoMFName failed')                    
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with another SoMFName failed' )
        t.setValues([SbName('1')])
        t.setValues(1,[SbName('2')])
        t.setValues(2,1,[SbName('3'), SbName('4')])
        self.failUnless(t.getValues() == ['1','2','3'],
                        'setValues with sequence of SbNames on SoMFName failed')                    

    def testMFNode(self):
        """check setValue(s) for SoMFNode"""
        t = SoMFNode()
        s = SoMFNode()
        c = SoMaterial()
        c.ref()
        c2 = SoTransform()
        c2.ref()
        t.setValue(c)
        self.failUnless(t.getValues() == [c], 
                        'setValue Node on SoMFNode failed')
        t.setValues([c,c2])
        self.failUnless(t.getValues() == [c,c2], 
                        'setValues on SoMFNode failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFNode on SoMFNode failed')
        t.setValues(2,1,[c,c2])
        self.failUnless(t.getValues() == [None,None,c],
                        'setValues with start, length and sequence on SoMFNode failed')         

    def testMFPath(self):
        """check setValue(s) for SoMFPath"""
        t = SoMFPath()
        s = SoMFPath()
        p = SoPath()
        p.ref()
        p2 = SoPath()
        p2.ref()
        t.setValue(p)
        self.failUnless(t.getValues() == [p], 
                        'setValue Path on SoMFPath failed')
        t.setValues([p,p2])
        self.failUnless(t.getValues() == [p,p2], 
                        'setValues on SoMFPath failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFPath on SoMFPath failed')
        t.setValues(2,1,[p,p2])
        self.failUnless(t.getValues() == [None,None,p],
                        'setValues with start, length and sequence on SoMFPath failed')         

    def testMFPlane(self):
        """check setValue(s) for SoMFPlane"""
        t = SoMFPlane()
        s = SoMFPlane()
        m = SbPlane(SbVec3f(1,0,0),0)
        m2 = SbPlane(SbVec3f(0,1,0),1)
        t.setValues([m])
        self.assertEqual(t.getValues(), [m])
        t.setValues(2,[m2,m2])
        self.assertEqual(t.getValues()[2:4],[m2,m2])
        t.setValues(1,1,[m2,m])
        self.assertEqual(t.getValues(), [m,m2,m2,m2])
        t.setValue(s)
        self.assertEqual(t.getValues(), s.getValues())
        t.setValue(m2)
        self.assertEqual(t.getValues(), [m2])
        t.setValues([])
        self.assertEqual(t.getValues(), [m2]) # appears to be the correct behaviour as in Coin :/

    def testMFRotation(self):
        """check setValue(s) for SoMFRotation"""
        t = SoMFRotation()
        s = SoMFRotation()
        m = SbRotation(1,0,0,0)
        m2 = SbRotation(0,0,0,1)
        t.setValues([m])
        self.assertEqual(t.getValues(), [m])
        t.setValues(2,[m2,m2])
        self.assertEqual(t.getValues()[2:4],[m2,m2])
        t.setValues(1,1,[m2,m])
        self.assertEqual(t.getValues(), [m,m2,m2,m2])
        t.setValue(s)
        self.assertEqual(t.getValues(), s.getValues())
        t.setValue(m2)
        self.assertEqual(t.getValues(), [m2])
        t.setValue(1,0,0,0)
        self.assertEqual(t.getValues(), [m])
        t.setValue([0,0,0,1])
        self.assertEqual(t.getValues(), [m2])
        t.setValues([])
        self.assertEqual(t.getValues(), [m2]) # appears to be the correct behaviour as in Coin :/

    def testMFTime(self):
        """check setValue(s) for SoMFTime"""
        t = SoMFTime()
        s = SoMFTime()
        t.setValues([SbTime(0.5),SbTime(2)])
        self.failUnless(t.getValues() == [SbTime(0.5),SbTime(2)],
                        'setValues with sequence on SoMFTime failed')
        t.setValues(2,[SbTime(0),SbTime(1)])
        self.failUnless(t.getValues() == [SbTime(0.5),SbTime(2),SbTime(0),SbTime(1)],
                        'setValues with start and sequence on SoMFTime failed')
        t.setValues(0,1,[SbTime(1.5),SbTime(0)])
        self.failUnless(t.getValues() == [SbTime(1.5),SbTime(2),SbTime(0),SbTime(1)],
                        'setValues with start, length and sequence on SoMFTime failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFTime on SoMFTime failed')
        t.setValue(SbTime(-0.5))
        self.failUnless(t.getValues() == [SbTime(-0.5)],
                        'setValue with single int on SoMFTime failed')
        t.setValues([])
        self.failUnless(t.getValues() == [SbTime(-0.5)],
                        'setValues with empty list on SoMFTime failed') 

    def testMFUInt32(self):
        """check setValue(s) for SoMFUInt32"""
        t = SoMFUInt32()
        s = SoMFUInt32()
        t.setValues([0,2])
        self.failUnless(t.getValues() == [0,2],
                        'setValues with sequence on SoMFUInt32 failed')
        t.setValues(2,[0,1])
        self.failUnless(t.getValues() == [0,2,0,1],
                        'setValues with start and sequence on SoMFUInt32 failed')
        t.setValues(0,1,[1,0])
        self.failUnless(t.getValues() == [1,2,0,1],
                        'setValues with start, length and sequence on SoMFUInt32 failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFUInt32 on SoMFUInt32 failed')
        t.setValue(0)
        self.failUnless(t.getValues() == [0],
                        'setValue with single int on SoMFUInt32 failed')
        t.setValues([])
        self.failUnless(t.getValues() == [0],
                        'setValues with empty list on SoMFUInt32 failed')

    def testMFUShort(self):
        """check setValue(s) for SoMFUShort"""
        t = SoMFUShort()
        s = SoMFUShort()
        t.setValues([0,2])
        self.failUnless(t.getValues() == [0,2],
                        'setValues with sequence on SoMFUShort failed')
        t.setValues(2,[0,1])
        self.failUnless(t.getValues() == [0,2,0,1],
                        'setValues with start and sequence on SoMFUShort failed')
        t.setValues(0,1,[1,0])
        self.failUnless(t.getValues() == [1,2,0,1],
                        'setValues with start, length and sequence on SoMFUShort failed')
        t.setValue(s)
        self.failUnless(t.getValues() == s.getValues(),
                        'setValue with other SoMFUShort on SoMFUShort failed')
        t.setValue(0)
        self.failUnless(t.getValues() == [0],
                        'setValue with single int on SoMFUShort failed')
        t.setValues([])
        self.failUnless(t.getValues() == [0],
                        'setValues with empty list on SoMFUShort failed')

    def testMFVec2f(self):
        """check setValue(s) for SoMFVec2f"""
        t = SoMFVec2f()
        s = SoMFVec2f()
        m = SbVec2f(1,0)
        m2 = SbVec2f(0,0)
        t.setValues([m])
        self.assertEqual(t.getValues(), [m])
        t.setValues(2,[m2,m2])
        self.assertEqual(t.getValues()[2:4],[m2,m2])
        t.setValues(1,1,[m2,m])
        self.assertEqual(t.getValues(), [m,m2,m2,m2])
        t.setValue(s)
        self.assertEqual(t.getValues(), s.getValues())
        t.setValue(m2)
        self.assertEqual(t.getValues(), [m2])
        t.setValues([])
        self.assertEqual(t.getValues(), [m2]) 
        t.setValues([[0,0],[1,0]])
        self.assertEqual(t.getValues(), [m2,m])
        t.setValue([0,0])
        self.assertEqual(t.getValues(), [m2])

    def testMFVec3f(self):
        """check setValue(s) for SoMFVec3f"""
        t = SoMFVec3f()
        s = SoMFVec3f()
        m = SbVec3f(1,0,0)
        m2 = SbVec3f(0,0,1)
        t.setValues([m])
        self.assertEqual(t.getValues(), [m])
        t.setValues(2,[m2,m2])
        self.assertEqual(t.getValues()[2:4],[m2,m2])
        t.setValues(1,1,[m2,m])
        self.assertEqual(t.getValues(), [m,m2,m2,m2])
        t.setValue(s)
        self.assertEqual(t.getValues(), s.getValues())
        t.setValue(m2)
        self.assertEqual(t.getValues(), [m2])
        t.setValues([])
        self.assertEqual(t.getValues(), [m2]) 
        t.setValues([[0,0,1],[1,0,0]])
        self.assertEqual(t.getValues(), [m2,m])
        t.setValue([0,0,1])
        self.assertEqual(t.getValues(), [m2])       

    def testMFVec4f(self):
        """check setValue(s) for SoMFVec4f"""
        t = SoMFVec4f()
        s = SoMFVec4f()
        m = SbVec4f(1,0,0,0)
        m2 = SbVec4f(0,0,1,0)
        t.setValues([m])
        self.assertEqual(t.getValues(), [m])
        t.setValues(2,[m2,m2])
        self.assertEqual(t.getValues()[2:4],[m2,m2])
        t.setValues(1,1,[m2,m])
        self.assertEqual(t.getValues(), [m,m2,m2,m2])
        t.setValue(s)
        self.assertEqual(t.getValues(), s.getValues())
        t.setValue(m2)
        self.assertEqual(t.getValues(), [m2])
        t.setValues([])
        self.assertEqual(t.getValues(), [m2]) 
        t.setValues([[0,0,1,0],[1,0,0,0]])
        self.assertEqual(t.getValues(), [m2,m])
        t.setValue([0,0,1,0])
        self.assertEqual(t.getValues(), [m2])        


class SbStringMethods(unittest.TestCase):
    """tests various string stuff"""
    def testCharConstructor(self):
        """check SbString(char *)"""
        s = SbString('hello')
        self.assertEqual(s.getString(), 'hello')

    def testCharLimits(self):
        """check SbString(char *, int begin, int end)"""
        s = SbString('hello', 1, 3)
        self.assertEqual(s.getString(), 'ell')

    def testOtherString(self):
        """check SbString(SbString)"""
        s = SbString('hello')
        t = SbString(s)
        self.assertEqual(s.getString(), t.getString())

    def testDigits(self):
        """check SbString(int)"""
        s = SbString(100)
        self.assertEqual(s.getString(), '100' )

class SbNameMethods(unittest.TestCase):
    """tests various Name stuff"""
    def testCharConstructor(self):
        """check SbName(char *)"""
        s = SbName('hello')
        self.assertEqual(s.getString(), 'hello')

    def testOtherName(self):
        """check SbName(SbName)"""
        s = SbName('hello')
        t = SbName(s)
        self.assertEqual(s.getString(), t.getString())

    def testOtherString(self):
        """check SbName(SbString)"""
        s = SbString('hello')
        t = SbName(s)
        self.assertEqual(s.getString(), t.getString())

    def testFromName(self):
        """check passing strings instead of SbName to SoType.fromName()"""
        t = SoType.fromName("SoCone")

class SbRotationMethods(unittest.TestCase):
    "tests various Rotation stuff"""
    def testAxisAngleCons(self):
        """check SbRotation(SbVector, float)"""
        r = SbRotation(SbVec3f(1,0,0), 3.14159264)
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,-4.3711388286737929e-008))

    def testFloat4Cons(self):
        """check SbRotation(float[4])"""
        r = SbRotation((1.0,0.0,0.0,0.0))
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,0.0))

    def testFloatsCons(self):
        """check SbRotation(float,float,float,float)"""
        r = SbRotation(1.0,0.0,0.0,0.0)
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,0.0))

    def testMatrixCons(self):
        """check SbRotation(SbMatrix)"""
        m = SbMatrix()
        m.setRotate(SbRotation(1.0,0.0,0.0,0.0))
        r = SbRotation(m)
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,0.0))

    # DISABLED until correctness of recently introduced precision
    # divergence in Coin is properly investigated. 20100426 tamer.
    # def testVecFTCons(self):
    #     """check SbRotation(SbVec3f, SbVec3f)"""
    #     r = SbRotation(SbVec3f(1,0,0), SbVec3f(0,1,0))
    #     r = r*r
    #     self.assertEqual(r.getValue(), (0.0,0.0,1.0,0.0))

    def testAxisAngleSet(self):
        """check SbRotation.setValue(SbVector, float)"""
        r = SbRotation()
        r.setValue(SbVec3f(1,0,0), 3.14159264)
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,-4.3711388286737929e-008))

    def testFloat4Set(self):
        """check SbRotation.setValue(float[4])"""
        r = SbRotation()
        r.setValue(1.0,0.0,0.0,0.0)
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,0.0))

    def testFloatsSet(self):
        """check SbRotation.setValue(float,float,float,float)"""
        r = SbRotation()
        r.setValue(1.0,0.0,0.0,0.0)
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,0.0))

    def testMatrixSet(self):
        """check SbRotation.setValue(SbMatrix)"""
        m = SbMatrix()
        m.setRotate(SbRotation(1.0,0.0,0.0,0.0))
        r = SbRotation()
        r.setValue(m)
        self.assertEqual(r.getValue(), (1.0,0.0,0.0,0.0))

    # DISABLED until correctness of recently introduced precision
    # divergence in Coin is properly investigated. 20100426 tamer.
    # def testVecFTSet(self):
    #     """check SbRotation.setValue(SbVec3f, SbVec3f)"""
    #     r = SbRotation()
    #     r.setValue(SbVec3f(1,0,0), SbVec3f(0,1,0))
    #     r = r*r
    #     self.assertEqual(r.getValue(), (0.0,0.0,1.0,0.0))

class SoBaseListMethods(unittest.TestCase):
    """tests various methods of SoBaseList"""
    def testGet(self):
        """check SoBaseList.get downcast type return"""
        l = SoBaseList()
        c = SoCone()
        l.append(c)
        self.assert_(isinstance(l.get(0), SoCone))

    def testItem(self):
        """check SoBaseList[] downcast type return"""
        l = SoBaseList()
        c = SoCone()
        l.append(c)
        self.assert_(isinstance(l[0], SoCone))

    def testSetItem(self):
        """check SoBaseList [0] = value support"""
        l = SoBaseList()
        f = SoCone()
        l.append(None) # note, that the set method underlying access does not expand!
        l[0] = f
        self.assert_(l[0].this == f.this)

    def testIteration(self):
        """check iteration support by all lists derived from SbPList"""
        l = SoBaseList()
        c = SoCone()
        l.append(c)
        for x in l:
            self.assert_(x.this == c.this)

class SoNodeListMethods(unittest.TestCase):
    """tests various methods of SoNodeList"""
    def testGet(self):
        """check SoNodeList.get() downcast type return"""
        l = SoNodeList()
        c = SoCone()
        l.append(c)
        self.assert_(isinstance(l.get(0), SoCone))

    def testItem(self):
        """check SoNodeList[] downcast type return"""
        l = SoNodeList()
        c = SoCone()
        l.append(c)
        self.assert_(isinstance(l[0], SoCone))

    def testSetItem(self):
        """check SoNodeList [0] = value support"""
        l = SoNodeList()
        c = SoCone()
        l.append(None) # note, that the set method underlying access does not expand!
        l[0] = c
        self.assert_(l[0].this == c.this)

class SoFieldListMethods(unittest.TestCase):
    """tests various methods of SoFieldList"""
    def testGet(self):
        """check SoFieldList.get() downcast type return"""
        l = SoFieldList()
        f = SoSFBool()
        l.append(f)
        self.assert_(isinstance(l.get(0), SoSFBool))

    def testItem(self):
        """check SoFieldList[] downcast type return"""
        l = SoFieldList()
        f = SoSFBool()
        l.append(f)
        self.assert_(isinstance(l[0], SoSFBool))

    def testSetItem(self):
        """check SoFieldList [0] = value support"""
        l = SoFieldList()
        f = SoSFBool()
        l[0] = f
        self.assert_(l[0] == f)

class FieldAssignment(unittest.TestCase):
    """tests field assignment for all kinds of types"""
    def testAssign(self):
        """check field assignment for all kinds of types"""
        n = SoCone()
        n.bottomRadius = 10
        self.assertEqual(n.bottomRadius.getValue(), 10 )
        n = SoInfo()
        n.string = "hello"
        self.assertEqual(n.string.getValue(), "hello" )

class SoFieldMethods(unittest.TestCase):
    """test SoField methods"""
    def testGet(self):
        """check SoField.get() ASCII field's value return"""
        s = SoSFVec3f()
        s.setValue(1,2,3)
        self.failUnless(s.get() == "1 2 3",
                        'SoField.get() failed')
                        
    def testLen(self):
        """check len() for SoMFields"""
        s = SoMFInt32()
        self.failUnless(len(s) == 0 == s.getNum(),
                        'len(s) on empty field failed')
        s.setValues([1,2,3])
        self.failUnless(len(s) == 3 == s.getNum(),
                        'len(s) on non-empty field failed')

    def testGet1(self):
        """check get1() for SoMFields"""
        s = SoMFInt32()
        s.setValues([1,2,3])
        self.failUnless(s.get1(1) == '2', 'get1(1) failed')
        
class SbTimeMethods(unittest.TestCase):
    """test SbTime methods"""
    def testConstructors(self):
        """check SbTime constructors"""
        t1 = SbTime()
        t2 = SbTime(12.005)
        t3 = SbTime(12, 5000)
        self.assertEqual( t2, t3 )
        
    def testSetValue(self):
        """check SbTime setValue methods"""
        t1 = SbTime(12.005)
        t2 = SbTime()
        t2.setValue(12.005)
        self.assertEqual(t1, t2)
        t2.setValue(12, 5000)
        self.assertEqual(t1, t2)

class OperatorTests(unittest.TestCase):
    """checks various operator overloaded methods"""
    def testEqNone(self):
        """check __eq__ operator None comparison"""
        self.failUnless(not (SoSeparator() == None))
        
    def testNqNone(self):
        """check __nq__ operator None comparison"""
        self.failUnless((SoSeparator() != None))
        
class InitTests(unittest.TestCase):
    """checks if all node packages are initialized at startup"""
    def testInitNodeKits(self):
        """check if SoNodeKit::init was called"""
        self.failUnless(SoShapeKit() is not None)

    def testInitInteraction(self):
        """check if interaction nodes (draggers etc) are initialized at startup"""
        self.failUnless(SoAntiSquish() is not None)

class NodeKitTests(unittest.TestCase):
    """checks API of SoBaseKit and related stuff"""
    def testRecursiveParts(self):
        """check if . returns a part not a nodefield"""
        s = SoShapeKit()
        self.failUnless(isinstance(s.appearance.material, SoMaterial))

class SoGroupMethods(unittest.TestCase):
    """checks methods and operators for SoGroup"""
    def testSequence(self):
        """check SoGroup sequence operator"""
        g = SoGroup()
        c = SoCone()
        g.addChild(c)
        self.assertEqual(len(g), 1)
        for x in g:
            self.assertEqual(x, c)

    def testContains(self):
        """check SoGroup containment operator in"""
        g = SoGroup()
        c = SoCone()
        c2 = SoCone()
        g.addChild(c)
        self.failUnless(c in g)
        self.failUnless(c2 not in g)

    def testAccess(self):
        """check SoGroup array access operator []"""
        g = SoGroup()
        c = SoCone()
        c2 = SoCone()
        g.addChild(c)
        g.addChild(c2)
        self.assertEqual(g[0], c)
        self.assertEqual(g[1], c2)
        try:
            g[2]
        except IndexError as e:
            test_sucessfully = True

    def testIAdd(self):
        g = SoGroup()
        c = SoCone()
        d = SoCube()
        g += [c, [g]]

class SbBaseClasses(unittest.TestCase):
    """checks methods and operators of and between various Sb* classes"""
    def testConstructors(self):
        v1 = SbVec3f()
        v2 = SbVec3f(1,2,3)
        v3 = SbVec3f([1,2,3])
        v4 = SbVec3f(v2)

        v1 = SbVec4f()
        v2 = SbVec4f(1,2,3,4)
        v3 = SbVec4f([1,2,3,4])
        v4 = SbVec4f(v2)

        v1 = SbVec3d()
        v2 = SbVec3d(1,2,3)
        v3 = SbVec3d([1,2,3])
        v4 = SbVec3d(v2)

        v1 = SbVec4d()
        v2 = SbVec4d(1,2,3,4)
        v3 = SbVec4d([1,2,3,4])
        v4 = SbVec4d(v2)

        m0 = SbMatrix()
        m1 = SbMatrix([[0,1,0,0], [0,0,1,0], [1,0,0,0], [0,0,0,1]])
        m2 = SbMatrix(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15)
        # the following is not part of Coin API, should it be supported ?
        #m3 = SbMatrix(m1)

        m0 = SbDPMatrix()
        m1 = SbDPMatrix([[0,1,0,0], [0,0,1,0], [1,0,0,0], [0,0,0,1]])
        m2 = SbDPMatrix(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15)
        # the following treats the SbMatrix as a sequence!
        #m3 = SbDPMatrix(SbMatrix())

    def testVecMatrixOperators(self):
        """tests operators between vec and matrix classes"""
        v = SbVec3f(1,0,0)
        m = SbMatrix([[0,1,0,0], [0,0,1,0], [1,0,0,0], [0,0,0,1]])
        self.assertEqual(m * v, SbVec3f(0,0,1))
        self.assertEqual(v * m, SbVec3f(0,1,0))
        v4 = SbVec4f(1,0,0,1)
        self.assertEqual(v4 * m, SbVec4f(0,1,0,1))

    def testVectorAsArrays(self):
        """tests various container operators of vectors"""

        d = {}
        d[2] = [1,2]
        d[3] = [1, 2, 3]
        d[4] = [1.0, 2.0, 3.0, 4.0]
        
        vecs = [SbVec2s(d[2]), SbVec2f(d[2]), SbVec2d(d[2]),
                SbVec3s(d[3]), SbVec3f(d[3]), SbVec3d(d[3]),
                SbVec4f(d[4]), SbVec4d(d[4])]

        for i,v in zip([2,2,2,3,3,3,4,4], vecs):
            self.assertEqual(i, len(v))
            self.assertEqual([x for x in v], d[i] )
            for j in range(i):
                self.assertEqual(d[i][j], v[j])

    def testVecdDBMatrixOperators(self):
        """tests operators between vec and matrix classes, double precision"""
        v = SbVec3d(1,0,0)
        m = SbDPMatrix([[0,1,0,0], [0,0,1,0], [1,0,0,0], [0,0,0,1]])
        self.assertEqual(m * v, SbVec3d(0,0,1))
        self.assertEqual(v * m, SbVec3d(0,1,0))
        # fails due to a bug in Coin3D, hopefully will be fixed soon
        v4 = SbVec4d(1,0,0,1)
        self.assertEqual(v4 * m, SbVec4d(0,1,0,1), 'bug in Coin3D, left in to check')

class SoPathTests(unittest.TestCase):
    """checks various methods for SoPath"""

    def setUp(self):
        self.nodes = [ SoSeparator() ]
        for i in range(3):
            child = SoGroup()
            self.nodes[-1].addChild(child)
            self.nodes.append(child)

    def tearDown(self):
        del self.nodes

    def testSoPathIterator(self):
        """check iterator in SoPath"""
        path = SoPath()
        for n in self.nodes:
            path.append(n)
        index = 0
        for n in path:
            self.assert_(n == self.nodes[index])
            index += 1
        for i in path.index():
            self.assertEqual(i, 0)

class SoInputTests(unittest.TestCase):
    """check method SoInput.setBuffer"""

    def setUp(self):
        self.text = "Separator {}"
        self.input = SoInput()

    def testSetBuffer(self):
        self.input.setBuffer(self.text)


class SbVecTests(unittest.TestCase):
    def setUp(self):
        self.sbvec3f = coin.SbVec3f(1, 1, 1)

    def multDivTest(self):
        self.assertEqual(self.sbvec3f, self.sbvec3f * 2. / 2.)
        # TODO: also support ints

class SbPLaneTests(unittest.TestCase):

    def setUp(self):
        self.a = coin.SbVec3f([0, 0, 0])
        self.b = coin.SbVec3f([0, 0, 1])
        self.c = coin.SbVec3f([0, 1, 0])
        self.line = coin.SbLine(a, b)
        self.plane1 = coin.SbPlane(b, a)
        self.plane2 = coin.SbPlane(c, a)

    def intersection(self):
        point = self.plane1.intersect(self.line)
        line = self.plane1.intersect(self.plane2)

if __name__ == "__main__":
    unittest.main(verbosity=4)


#!/usr/bin/env python

###
# Copyright (c) 2002-2009 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# Pivy distutils setup script.
#

"""Pivy is a Coin binding for Python. Coin is a high-level 3D graphics
library with a C++ Application Programming Interface. Coin uses
scene-graph data structures to render real-time graphics suitable for
mostly all kinds of scientific and engineering visualization
applications.
"""
from __future__ import print_function

###
# Setup file for the Pivy distribution.
#
import glob
import os
import shutil
import subprocess
import sys

from distutils.command.build import build
from distutils.command.clean import clean
from distutils.command.install import install
from distutils.core import setup
from distutils.extension import Extension
from distutils import sysconfig

# if we are on a Gentoo box salute the chap and output stuff in nice colors
# Gentoo is Python friendly, so be especially friendly to them! ;)
try:
    from portage.output import green, blue, turquoise, red, yellow
    print(red("Oooh, it's a Gentoo! Nice nice! tuhtah salutes you! :)"))
except:
    try:
        from colorama import Fore, Style

        def red(text): return Fore.RED + text + Style.RESET_ALL

        def green(text): return Fore.GREEN + text + Style.RESET_ALL

        def blue(text): return Fore.BLUE + text + Style.RESET_ALL

        def turquoise(text): return Fore.CYAN + text + Style.RESET_ALL

        def yellow(text): return Fore.YELLOW + text + Style.RESET_ALL
    except:
        def red(text): return text

        def green(text): return text

        def blue(text): return text

        def turquoise(text): return text

        def yellow(text): return text

PIVY_CLASSIFIERS = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: Unix
Programming Language :: Python
Topic :: Multimedia :: Graphics
Topic :: Multimedia :: Graphics :: 3D Modeling
Topic :: Multimedia :: Graphics :: 3D Rendering
Topic :: Software Development :: Libraries :: Python Modules
"""

__dir_name__ = os.path.dirname(__file__)
pivy_dir = os.path.join(__dir_name__, "pivy")
sys.path.append(pivy_dir)
import pivy_meta
PIVY_VERSION = pivy_meta.__version__
sys.path.pop(-1)

class pivy_build(build):
    PIVY_SNAKES = r"""
                            _____
                        .-'`     '.                                     
                     __/  __       \                                   
                    /  \ /  \       |    ___                          
                   | /`\| /`\|      | .-'  /^\/^\                   
                   | \(/| \(/|      |/     |) |)|                     
                  .-\__/ \__/       |      \_/\_/__..._             
          _...---'-.                /   _              '.               
         /,      ,             \   '|  `\                \           
        | ))     ))           /`|   \    `.       /)  /) |             
        | `      `          .'       |     `-._         /               
        \                 .'         |     ,_  `--....-'               
         `.           __.' ,         |     / /`'''`                     
           `'-.____.-' /  /,         |    / /                           
               `. `-.-` .'  \        /   / |                           
                 `-.__.'|    \      |   |  |-.                         
                    _.._|     |     /   |  |  `'.                       
              .-''``    |     |     |   /  |     `-.                    
           .'`         /      /     /  |   |        '.                  
         /`           /      /     |   /   |\         \               
        /            |      |      |   |   /\          |               
       ||            |      /      |   /     '.        |                
       |\            \      |      /   |       '.      /              
       \ `.           '.    /      |    \        '---'/               
        \  '.           `-./        \    '.          /                
         '.  `'.            `-._     '.__  '-._____.'--'''''--.         
           '-.  `'--._          `.__     `';----`              \       
              `-.     `-.          `.''```                     ;        
                 `'-..,_ `-.         `'-.                     /         
                        '.  '.           '.                 .'          


                            ~~~ HISSSSSSSSSS ~~~
                           Welcome to Pivy %s!
                 Building Pivy has never been so much fun!

    """ % PIVY_VERSION

    pivy_header_include = """\
#ifdef __PIVY__
%%include %s
#endif

"""

    SWIG = ((sys.platform == "win32" and "swig.exe") or "swig")

    SWIG_SUPPRESS_WARNINGS = "-w302,306,307,312,314,325,361,362,467,389,503,509,510"
    SWIG_PARAMS = "-c++ -python -includeall -modern -D__PIVY__ " + \
                  "-I. -Ifake_headers -I\"%s\" %s -o %s_wrap.cpp " + \
                  "interfaces" + os.sep + "%s.i"
    if sys.version_info.major >= 3:
        SWIG_PARAMS = '-py3 ' + SWIG_PARAMS


    # TODO: add command line arguments to tell distutils which library should be build
    SOGUI = ['soqt']  # stop supporting soxt, sogtk, and sowin to simplify the setup
    coin_interface = 'coin'
    soqt_interface = 'soqt'

    if sys.version_info.major < 3:
        coin_interface = 'coin2'
        soqt_interface = 'soqt2'

    MODULES = {
        'coin': ['_coin', 'COIN', 'pivy.', coin_interface],
        'soqt': ['gui._soqt', 'SOQT', 'pivy.gui.', soqt_interface]
    }

    SUPPORTED_SWIG_VERSIONS = ['3.0.8', '3.0.10', '3.0.12', '4.0.0']
    SWIG_VERSION = ""
    SWIG_COND_SYMBOLS = []
    CXX_INCS = "-Iinterfaces "
    CXX_LIBS = ""

    ext_modules = []
    py_modules = ['pivy.quarter.ContextMenu',
                  'pivy.quarter.ImageReader',
                  'pivy.quarter.QuarterWidget',
                  'pivy.quarter.SensorManager',
                  'pivy.quarter.SignalThread',
                  'pivy.quarter.devices.DeviceHandler',
                  'pivy.quarter.devices.DeviceManager',
                  'pivy.quarter.devices.KeyboardHandler',
                  'pivy.quarter.devices.MouseHandler',
                  'pivy.quarter.eventhandlers.DragDropHandler',
                  'pivy.quarter.eventhandlers.EventHandler',
                  'pivy.quarter.eventhandlers.EventManager',
                  'pivy.quarter.plugins.designer.python.PyQuarterWidgetPlugin',
                  'pivy.utils',
                  'pivy.graphics.colors',
                  'pivy.graphics.mesh',
                  'pivy.graphics.plot',
                  'pivy.graphics.viewer']

    def check_with_cmake(self):
        dirname = os.path.dirname(__file__)
        cmake_command = ['cmake', dirname]
        try:
            cmake_command += ['-G', os.environ['GENERATOR']]
        except KeyError:
            pass
        print(yellow('calling: ' + cmake_command[0] + ' ' + cmake_command[1]))
        cmake = subprocess.Popen(cmake_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        cmake_out, cmake_err = cmake.communicate()
        coin_vars = ['COIN_FOUND', 'COIN_VERSION', 'COIN_INCLUDE_DIR', 'COIN_LIB_DIR']
        soqt_vars = ['SOQT_FOUND', 'SOQT_VERSION', 'SOQT_INCLUDE_DIR', 'SOQT_LIB_DIR']
        config_dict = {}
        print(yellow(cmake_out.decode("utf-8")))
        print(red(cmake_err.decode("utf-8")))
        if cmake.returncode == 0:
            for line in cmake_out.decode("utf-8").split("\n"):
                for var in coin_vars + soqt_vars:
                    if var in line:
                        line = (line
                                .replace('-- ' + var, '')
                                .replace(': ', '')
                                .replace('\n', ''))
                        if "INCLUDE" in var:
                            line = line.replace(';', '\" -I\"')
                        config_dict[var] = line

        print(yellow('\nchecking for COIN via cmake'))
        for key in coin_vars:
            if key in config_dict:
                print(blue(key + ': ' + config_dict[key]))

        print(yellow('\nchecking for SOQT via cmake'))
        for key in soqt_vars:
            if key in config_dict:
                print(blue(key + ': ' + config_dict[key]))

        if config_dict.get('SOQT_FOUND', 'false') == 'false':
            pivy_build.MODULES.pop('soqt')
            print(red("\ndisable soqt, because cmake couldn't find it"))
        else:
            try:
                import qtinfo
                self.QTINFO = qtinfo.QtInfo()
            except Exception as e:
                import traceback
                print(red("\ndisable soqt, because there was a problem running qtinfo (needs qmake)"))
                print(red("-" * 60))
                print(red(traceback.print_exc()))
                print(red("-" * 60))
                pivy_build.MODULES.pop('soqt')

        self.cmake_config_dict = config_dict
        if self.cmake_config_dict.get('COIN_FOUND', 'false') == 'false':
            raise(RuntimeError('coin was not found, but you need coin to build pivy'))

    def do_os_popen(self, cmd):
        "return the output of a command in a single line"
        fd = os.popen(cmd)
        lines = fd.readlines()
        for i in range(len(lines)):
            lines[i] = lines[i].strip()
        lines = " ".join(lines)
        fd.close()
        return lines

    def check_cmd_exists(self, cmd):
        "return the path of the specified command if it exists"
        print(blue("Checking for %s..." % cmd))
        for path in os.environ['PATH'].split(os.path.pathsep):
            if os.path.exists(os.path.join(path, cmd)):
                print(blue("'%s'" % os.path.join(path, cmd)))
                return 1
        print(red("not found."))
        return 0

    def check_python_version(self):
        "check the Python version"
        print(blue("Python version...%s" % sys.version.split(" ")[0]))
        if int(sys.version[0]) < 2:
            print(red("Pivy only works with Python versions >= 2.0."))
            sys.exit(1)

    def check_coin_version(self):
        '''
        check the Coin version
        '''
        print(yellow('\ncheck_coin_version is not supported in this version'))
        print(yellow('coin-bindings are build by default'))
        print(yellow('checks have been disabled because of missing config files'))
        print(yellow('make sure you have installed the coin library + headers!'))
        return #TODO

        if sys.platform == "win32":
            return
        if not self.check_cmd_exists("coin-config"):
            sys.exit(1)
        print(blue("Coin version..."))
        version = self.do_os_popen("coin-config --version")
        print(blue("%s" % version))
        # if not version.startswith('3'):
        #     print(yellow("** Warning: Pivy has only been tested with Coin "
        #                  "versions Coin-dev 3."))

    def check_simvoleon_version(self):
        '''return if SIMVoleon is available and check the version'''
        print(yellow('\ncheck_simvoleon_version is not supported in this version'))
        return #TODO

        if sys.platform == "win32" or not self.check_cmd_exists("simvoleon-config"):
            self.MODULES.pop('simvoleon', None)
            return False

        print(blue("SIMVoleon version..."))
        version = self.do_os_popen("simvoleon-config --version")
        print(blue("%s" % version))
        if not version.startswith('2.0'):
            print(yellow("** Warning: Pivy has only been tested with SIMVoleon "
                         "versions 2.0.x."))
        return True

    def check_gui_bindings(self):
        '''check for availability of SoGui bindings and removes the not available ones'''

        print(yellow('\ncheck_gui_bindings is not supported in this version'))
        print(yellow('soqt is build by default'))
        print(yellow('make sure you have installed the soqt library + headers\n'))
        return #TODO

        if sys.platform == "_win32":
            self.MODULES.pop('soxt', None)
            self.MODULES.pop('sogtk', None)
            print(blue("Checking for SoWin..."))
            if not os.path.exists(os.path.join(os.getenv("COINDIR"), "include", "Inventor", "Win", "SoWin.h")):
                self.MODULES.pop('sowin', None)
                print(red("COINDIR\\include\\Inventor\\Win\\SoWin.h not found. (SoWin bindings won't be built)"))
            print(blue("Checking for QTDIR environment variable..."))
            if os.getenv("QTDIR"):
                print(blue(os.getenv("QTDIR")))
            else:
                self.MODULES.pop('soqt', None)
                print(red("not set. (SoQt bindings won't be built)"))
        else:
            for gui in self.SOGUI:
                if gui not in self.MODULES:
                    continue
                gui_config_cmd = self.MODULES[gui][1]
                if not self.check_cmd_exists(gui_config_cmd):
                    self.MODULES.pop(gui, None)
                else:
                    print(blue("Checking for %s version..." % gui))
                    version = self.do_os_popen("%s --version" % gui_config_cmd)
                    print(blue("%s" % version))

    def get_coin_features(self):
        '''
        set the global variable SWIG_COND_SYMBOLS needed for conditional wrapping
        '''
        print(yellow('\ncoin-features are not supported in this version'))
        return #TODO

        if sys.platform == "win32":
            return
        print(blue("Checking for Coin features..."))
        if not os.system("coin-config --have-feature 3ds_import"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_3DS_IMPORT")
            print(green("3ds import "))

        if not os.system("coin-config --have-feature vrml97"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_VRML97")
            print(green("vrml97 "))

        if not os.system("coin-config --have-feature sound"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_SOUND")
            print(green("sound "))

        if not os.system("coin-config --have-feature superglu"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_SUPERGLUE")
            print(green("superglu "))

        if not os.system("coin-config --have-feature threads"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_THREADS")
            print(green("threads "))

        if not os.system("coin-config --have-feature threadsafe"):
            self.SWIG_COND_SYMBOLS.append("-DHAVE_FEATURE_THREADSAFE")
            print(green("threadsafe "))

        print()

    def check_swig_version(self, swig):
        "check for the swig version"
        global SWIG_VERSION
        if not self.check_cmd_exists(swig):
            # on some systems there is only a swig3.0 so check for this and
            # set SWIG to "swig3.0"
            swig = "swig3.0"
            if not self.check_cmd_exists(swig):
                sys.exit(1)
            else:
                self.SWIG = swig
        print(blue("Checking for SWIG version..."))
        p = subprocess.Popen("%s -version" % swig,
                             shell=True, stdout=subprocess.PIPE)
        version = str(p.stdout.readlines()[1].strip()).split(" ")[2]
        if version[-1] == "'":
            version = version[:-1]
        p.stdout.close()
        print(blue("%s" % version))
        SWIG_VERSION = version
        if version not in self.SUPPORTED_SWIG_VERSIONS:
            print(yellow("Warning: Pivy has only been tested with the following " +
                         "SWIG versions: %s." % " ".join(self.SUPPORTED_SWIG_VERSIONS)))

    def copy_and_swigify_headers(self, includedir, dirname, files):
        """Copy the header files to the local include directories. Add an
        #include line at the beginning for the SWIG interface files..."""
        for file in files:
            if not os.path.isfile(os.path.join(dirname, file)):
                continue

            if file[-2:] == ".i":
                file_i = os.path.join(dirname, file)
                file_h = os.path.join(dirname, file)[:-2] + ".h"

                if (not os.path.exists(file_h) and os.path.exists(os.path.join(includedir, file_h))):
                    shutil.copyfile(os.path.join(includedir, file_h), file_h)
                    sys.stdout.write(' ' + turquoise(file_h))
                    fd = open(file_h, 'r+')
                    contents = fd.readlines()

                    ins_line_nr = -1
                    for line in contents:
                        ins_line_nr += 1
                        if line.find("#include ") != -1:
                            break

                    if ins_line_nr != -1:
                        contents.insert(ins_line_nr, self.pivy_header_include % (file_i))
                        fd.seek(0)
                        fd.writelines(contents)
                    else:
                        print(blue("[") + red("failed") + blue("]"))
                        sys.exit(1)
                    fd.close
            # fixes for SWIG 1.3.21 and upwards
            # (mostly workarounding swig's preprocessor "function like macros"
            # preprocessor bug when no parameters are provided which then results
            # in no constructors being created in the wrapper)
            elif file[-4:] == ".fix":
                sys.stdout.write(' ' + red(os.path.join(dirname, file)[:-4]))
                shutil.copyfile(os.path.join(dirname, file),
                                os.path.join(dirname, file)[:-4])
            # had to introduce this because windows is a piece of crap
            elif sys.platform == "win32" and file[-6:] == ".win32":
                sys.stdout.write(' ' + red(os.path.join(dirname, file)[:-6]))
                shutil.copyfile(os.path.join(dirname, file),
                                os.path.join(dirname, file)[:-6])

    def pivy_configure(self):
        '''
        configure pivy
        '''
        print(turquoise(self.PIVY_SNAKES))
        print(blue("Platform...%s" % sys.platform))
        self.check_python_version()
        self.check_swig_version(self.SWIG)
        # self.check_coin_version()

        self.check_with_cmake()

        # TODO: find a way to enable coin-features
        self.get_coin_features()
        # if self.SOGUI:
        #     self.check_gui_bindings()

        if 'simvoleon' in self.MODULES and self.check_simvoleon_version():
            if sys.platform == "win32":
                INCLUDE_DIR = os.getenv("SIMVOLEONDIR") + "\\include"
            else:
                INCLUDE_DIR = self.do_os_popen("simvoleon-config --includedir")

            sys.stdout.write(blue("Preparing") + green(" VolumeViz ") + blue("headers:"))
            dir_gen = os.walk("VolumeViz", INCLUDE_DIR)
            for _dir, _, names in dir_gen:
                self.copy_and_swigify_headers(INCLUDE_DIR, _dir, names)
            print(green("."))

        # if sys.platform == "win32":
        #     INCLUDE_DIR = os.path.join(os.getenv("COINDIR"), "include")
        # else:
            # INCLUDE_DIR = sysconfig.get_config_var("INCLUDEDIR")

        # TODO: Check on win
        INCLUDE_DIR = self.cmake_config_dict['COIN_INCLUDE_DIR']

        sys.stdout.write(blue("Preparing") + green(" Inventor ") + blue("headers:"))
        dir_gen = os.walk("Inventor", INCLUDE_DIR)
        for _dir, _, names in dir_gen:
            self.copy_and_swigify_headers(INCLUDE_DIR, _dir, names)
        print(green("."))

    def swig_generate(self):
        "build all available modules"

        def quote(s): return '"' + s + '"'

        def win_quote(s):
            if sys.platform == 'win32':
                return '"' + s + '"'
            return s

        for module in self.MODULES:
            module_name = self.MODULES[module][0]
            config_cmd = self.MODULES[module][1]
            module_pkg_name = self.MODULES[module][2]
            mod_hack_name = self.MODULES[module][3]
            mod_out_prefix = module_pkg_name.replace('.', os.sep) + module
            try:
                CPP_FLAGS = os.environ["PIVY_CPP_FLAGS"]
            except KeyError:
                CPP_FLAGS = ""

            if sys.platform == "_win32":  # this should never happen
                INCLUDE_DIR = os.path.join(os.getenv("COINDIR"), "include")
                CPP_FLAGS += "-I" + quote(INCLUDE_DIR) + " " + \
                            "-I" + quote(os.path.join(os.getenv("COINDIR"), "include", "Inventor", "annex")) + \
                            " /DCOIN_DLL /wd4244 /wd4049"
                # aquire highest non-debug Coin library version
                try:
                    LDFLAGS_LIBS = quote(
                        max(glob.glob(os.path.join(os.getenv("COINDIR"), "lib", "coin?.lib")))) + " "
                # with cmake the coin library is named Coin4.lib
                except ValueError:
                    LDFLAGS_LIBS = quote(
                        max(glob.glob(os.path.join(os.getenv("COINDIR"), "lib", "Coin?.lib")))) + " "

                if module == "sowin":
                    CPP_FLAGS += " /DSOWIN_DLL"
                    LDFLAGS_LIBS += quote(os.path.join(os.getenv("COINDIR"), "lib", "sowin1.lib"))
                elif module == "soqt":
                    CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\"  /DSOQT_DLL"
                    if os.path.isdir(os.getenv("QTDIR") + "\\include\Qt\""):
                        CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\Qt\""
                        LDFLAGS_LIBS += os.path.join(os.getenv("COINDIR"), "lib", "soqt1.lib") + " "
                    else:
                        # workaround for conda qt4:
                        CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\qt\Qt\""
                        CPP_FLAGS += " -I" + '"' + os.getenv("QTDIR") + "\\include\qt\""
                        LDFLAGS_LIBS += os.path.join(os.getenv("COINDIR"), "lib", "SoQt.lib") + " "
            else:
                INCLUDE_DIR = self.cmake_config_dict[config_cmd + '_INCLUDE_DIR']
                LIB_DIR = self.cmake_config_dict[config_cmd + '_LIB_DIR']
                if sys.platform == 'win32':
                    _INCLUDE_DIR = INCLUDE_DIR
                else:
                    # replace all quotes from INCLUDE_DIR
                    _INCLUDE_DIR = INCLUDE_DIR.replace('"', "")
                CPP_FLAGS += ' -I' + _INCLUDE_DIR
                CPP_FLAGS += ' -I' + os.path.join(_INCLUDE_DIR, 'Inventor', 'annex')
                if sys.platform == 'win32': 
                    CPP_FLAGS += " /DCOIN_DLL /wd4244 /wd4049"
                    LDFLAGS_LIBS = quote(max(glob.glob(os.path.join(LIB_DIR, "Coin?.lib")))) + " "
                else:
                    CPP_FLAGS += " -Wno-unused -Wno-maybe-uninitialized"
                    LDFLAGS_LIBS = ' -L' + self.cmake_config_dict[config_cmd + '_LIB_DIR']

                if module == "soqt":
                    CPP_FLAGS += ' -I' + win_quote(self.QTINFO.getHeadersPath())
                    CPP_FLAGS += ' -I' + win_quote(os.path.join(self.QTINFO.getHeadersPath(), 'QtCore'))
                    CPP_FLAGS += ' -I' + win_quote(os.path.join(self.QTINFO.getHeadersPath(), 'QtGui'))
                    CPP_FLAGS += ' -I' + win_quote(os.path.join(self.QTINFO.getHeadersPath(), 'QtOpenGL'))
                    CPP_FLAGS += ' -I' + win_quote(os.path.join(self.QTINFO.getHeadersPath(), 'QtWidgets'))
                    if sys.platform == 'win32':
                        LDFLAGS_LIBS += " " + quote(max(glob.glob(os.path.join(LIB_DIR, "SoQt?.lib")))) + " "
                        CPP_FLAGS += " /DSOQT_DLL"
                    else:
                        LDFLAGS_LIBS += ' -lSoQt'
                
                if module == "coin":
                    if sys.platform == 'win32':
                        pass
                    else:
                        LDFLAGS_LIBS += ' -lCoin'

            if not os.path.isfile(mod_out_prefix + "_wrap.cpp"):
                print(red("\n=== Generating %s_wrap.cpp for %s ===\n" %
                          (mod_out_prefix, module)))
                print(blue(self.SWIG + " " + self.SWIG_SUPPRESS_WARNINGS + " " + self.SWIG_PARAMS %
                           (INCLUDE_DIR,
                            self.CXX_INCS,
                            mod_out_prefix, module)))
                if os.system(self.SWIG + " " + self.SWIG_SUPPRESS_WARNINGS + " " + self.SWIG_PARAMS %
                             (INCLUDE_DIR,
                              self.CXX_INCS,
                              mod_out_prefix, mod_hack_name)):
                    print(red("SWIG did not generate wrappers successfully! ** Aborting **"))
                    sys.exit(1)
            else:
                print(red("=== %s_wrap.cpp for %s already exists! ===" %
                          (mod_out_prefix, module_pkg_name + module)))

            self.ext_modules.append(Extension(module_name, [mod_out_prefix + "_wrap.cpp"],
                                              extra_compile_args=(
                                                  self.CXX_INCS + CPP_FLAGS).split(),
                                              extra_link_args=(self.CXX_LIBS + LDFLAGS_LIBS).split()))

    def run(self):
        "the entry point for the distutils build class"
        # if sys.platform == "win32" and not os.getenv("COINDIR"):
        #     print("Please set the COINDIR environment variable to your Coin root directory! ** Aborting **")
        #     sys.exit(1)

        self.pivy_configure()
        self.swig_generate()

        for cmd_name in self.get_sub_commands():
            self.run_command(cmd_name)


class pivy_clean(clean):
    pivy_path = 'pivy' + os.sep
    gui_path = 'pivy' + os.sep + 'gui' + os.sep
    REMOVE_FILES = (pivy_path + '__init__.pyc', gui_path + '__init__.pyc',
                    pivy_path + 'coin_wrap.cpp', pivy_path + 'coin2_wrap.cpp', pivy_path + 'coin.py', pivy_path + 'coin.pyc',
                    pivy_path + 'simvoleon_wrap.cpp', pivy_path + 'simvoleon.py', pivy_path + 'simvoleon.pyc',
                    gui_path + 'soqt_wrap.cpp', gui_path + 'soqt.py', gui_path + 'soqt.pyc',
                    gui_path + 'sogtk_wrap.cpp', gui_path + 'sogtk.py', gui_path + 'sogtk.py',
                    gui_path + 'soxt_wrap.cpp', gui_path + 'soxt.py', gui_path + 'soxt.pyc',
                    gui_path + 'sowin_wrap.cpp', gui_path + 'sowin.py', gui_path + 'sowin.pyc',
                    pivy_path + 'sogui.pyc')

    def remove_headers(self, arg, dirname, files):
        "remove the coin headers from the pivy Inventor directory"
        for file in files:
            if not os.path.isfile(os.path.join(dirname, file)) or file[-2:] != ".h":
                continue
            sys.stdout.write(' ' + turquoise(os.path.join(dirname, file)))
            os.remove(os.path.join(dirname, file))

    def run(self):
        "the entry point for the distutils clean class"
        sys.stdout.write(blue("Cleaning headers:"))
        dir_gen = os.walk("Inventor")
        for _dir, _, names in dir_gen:
            self.remove_headers(None, _dir, names)

        dir_gen = os.walk("VolumeViz")
        for _dir, _, names in dir_gen:
            self.remove_headers(None, _dir, names)

        self.remove_cmake()

        # remove the SWIG generated wrappers
        for wrapper_file in self.REMOVE_FILES:
            if os.path.isfile(wrapper_file):
                sys.stdout.write(' ' + turquoise(wrapper_file))
                os.remove(wrapper_file)
        print(green("."))

        clean.run(self)

    def remove_cmake(self):
        if os.path.isdir('CMakeFiles'):
            shutil.rmtree('CMakeFiles')
        if os.path.isfile('CMakeCache.txt'):
            os.remove('CMakeCache.txt')


for i in reversed(list(range(len(sys.argv)))):
    if sys.argv[i][:10] == "--without-":
        pivy_build.MODULES.pop(sys.argv[i][10:], None)
        del sys.argv[i]

 
setup(name="Pivy",
      version=PIVY_VERSION,
      description="A Python binding for Coin",
      long_description=__doc__,
      author="Tamer Fahmy",
      author_email="tamer@sim.no",
      download_url="https://github.com/coin3d/pivy/releases",
      url="http://pivy.coin3d.org/",
      cmdclass={'build': pivy_build,
                  'clean': pivy_clean},
      ext_package='pivy',
      ext_modules=pivy_build.ext_modules,
      py_modules=pivy_build.py_modules,
      packages=['pivy', 'pivy.gui'],
      classifiers=[_f for _f in PIVY_CLASSIFIERS.split("\n") if _f],
      license="BSD License",
      platforms=['Any']
      )



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 12, example 3.
#
# Alarm sensor that raises a flag after 10 minutes
#

import sys

from pivy.coin import *
from pivy.sogui import *

###########################################################
# CODE FOR The Inventor Mentor STARTS HERE

def raiseFlagCallback(flagAngleXform, sensor):
    # We know that flagAngleXform is an autocasted SoTransform node
    # Rotate flag by 90 degrees about the Z axis:
    flagAngleXform.rotation.setValue(SbVec3f(0,0,1), M_PI/2)

# CODE FOR The Inventor Mentor ENDS HERE
###########################################################


def main():
    myWindow = SoGui.init(sys.argv[0]) # pass the app name
    if myWindow == None: sys.exit(1)

###########################################################
# CODE FOR The Inventor Mentor STARTS HERE

    flagXform = SoTransform()

    # Create an alarm that will call the flag-raising callback:
    myAlarm = SoAlarmSensor(raiseFlagCallback, flagXform)
    myAlarm.setTimeFromNow(12.0)  # 12 seconds
    myAlarm.schedule()

# CODE FOR The Inventor Mentor ENDS HERE
###########################################################

    root = SoSeparator()
    root.addChild(flagXform)
    myCone = SoCone()
    myCone.bottomRadius = 0.1
    root.addChild(myCone)

    myViewer = SoGuiExaminerViewer(myWindow)

    # Put our scene in myViewer, change the title
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Raise The Cone")
    myViewer.show()

    SoGui.show(myWindow)  # Display main window
    SoGui.mainLoop()      # Main Inventor event loop

if __name__ == "__main__":
    main()



import sys

from PySide2 import QtWidgets
from pivy import coin, quarter


def main():
    app = QtWidgets.QApplication(sys.argv)

    root = coin.SoSeparator()
    mat = coin.SoMaterial()
    mat.diffuseColor.setValue(coin.SbColor(0.8, 1, 0.8))
    mat.specularColor.setValue(coin.SbColor(1, 1, 1))
    mat.shininess.setValue(1.0)
    mat.transparency.setValue(0.9)
    root.addChild(mat)
    root.addChild(coin.SoSphere())
    root.addChild(coin.SoCube())

    viewer = quarter.QuarterWidget()
    viewer.setSceneGraph(root)

    viewer.setWindowTitle("minimal")
    viewer.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



import sys
from PySide2.QtWidgets import QApplication
from PySide2.QtGui import QColor
from pivy import quarter, coin, graphics, utils

class ConnectionMarker(graphics.Marker):
    def __init__(self, points):
        super(ConnectionMarker, self).__init__(points, True)


class ConnectionPolygon(graphics.Polygon):
    std_col = "green"
    def __init__(self, markers):
        super(ConnectionPolygon, self).__init__(
            sum([m.points for m in markers], []), True)
        self.markers = markers

        for m in self.markers:
            m.on_drag.append(self.update_polygon)

    def update_polygon(self):
        self.points = sum([m.points for m in self.markers], [])

    @property
    def drag_objects(self):
        return self.markers

    def check_dependency(self):
        if any([m._delete for m in self.markers]):
            self.delete()


class ConnectionLine(graphics.Line):
    def __init__(self, markers):
        super(ConnectionLine, self).__init__(
            sum([m.points for m in markers], []), True)
        self.markers = markers
        for m in self.markers:
            m.on_drag.append(self.update_line)

    def update_line(self):
        self.points = sum([m.points for m in self.markers], [])

    @property
    def drag_objects(self):
        return self.markers

    def check_dependency(self):
        if any([m._delete for m in self.markers]):
            self.delete()


def main():
    app = QApplication(sys.argv)
    utils.addMarkerFromSvg("test.svg", "CUSTOM_MARKER",  40)
    viewer = quarter.QuarterWidget()
    root = graphics.InteractionSeparator(viewer.sorendermanager)
    root.pick_radius = 40

    m1 = ConnectionMarker([[-1, -1, -1]])
    m2 = ConnectionMarker([[-1,  1, -1]])
    m3 = ConnectionMarker([[ 1,  1, -1]])
    m4 = ConnectionMarker([[ 1, -1, -1]])

    m5 = ConnectionMarker([[-1, -1,  1]])
    m6 = ConnectionMarker([[-1,  1,  1]])
    m7 = ConnectionMarker([[ 1,  1,  1]])
    m8 = ConnectionMarker([[ 1, -1,  1]])

    points = [m1, m2, m3, m4, m5, m6, m7, m8]

    l01 = ConnectionLine([m1, m2])
    l02 = ConnectionLine([m2, m3])
    l03 = ConnectionLine([m3, m4])
    l04 = ConnectionLine([m4, m1])

    l05 = ConnectionLine([m5, m6])
    l06 = ConnectionLine([m6, m7])
    l07 = ConnectionLine([m7, m8])
    l08 = ConnectionLine([m8, m5])

    l09 = ConnectionLine([m1, m5])
    l10 = ConnectionLine([m2, m6])
    l11 = ConnectionLine([m3, m7])
    l12 = ConnectionLine([m4, m8])

    lines = [l01, l02, l03, l04, l05, l06, l07, l08, l09, l10, l11, l12]

    p1 = ConnectionPolygon([m1, m2, m3, m4])
    p2 = ConnectionPolygon([m8, m7, m6, m5])
    p3 = ConnectionPolygon([m5, m6, m2, m1])
    p4 = ConnectionPolygon([m6, m7, m3, m2])
    p5 = ConnectionPolygon([m7, m8, m4, m3])
    p6 = ConnectionPolygon([m8, m5, m1, m4])

    polygons = [p1, p2, p3, p4, p5, p6]
    root += points + lines + polygons
    root.register()

    viewer.setSceneGraph(root)
    viewer.setBackgroundColor(QColor(255, 255, 255))
    viewer.setWindowTitle("minimal")
    viewer.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 7, example 1.
#
# This example displays a textured cube (default 
# texture coords).
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoSeparator()

    # Choose a texture 
    rock = SoTexture2()
    root.addChild(rock)
    rock.filename = "brick.1.rgb"

    # Make a cube
    root.addChild(SoCube())

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Default Texture Coords")

    # In Inventor 2.1, if the machine does not have hardware texture
    # mapping, we must override the default drawStyle to display textures.
    # myViewer.setDrawStyle(SoGuiViewer.STILL, SoGuiViewer.VIEW_AS_IS)
    
    myViewer.show()
    myViewer.viewAll()
    
    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor,
# chapter 4, example 2.
#
# Lights example.  
# Read in an object from a file.
# Use the ExaminerViewer to view it with two light sources.
# The red directional light doesn't move; the green point 
# light is moved back and forth using a shuttle node.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])
    if myWindow == None: sys.exit(1)

    root = SoSeparator()

    # Add a directional light
    myDirLight = SoDirectionalLight()
    myDirLight.direction = (0, -1, -1)
    myDirLight.color = (1, 0, 0)
    root.addChild(myDirLight)

    # Put the shuttle and the light below a transform separator.
    # A transform separator pushes and pops the transformation 
    # just like a separator node, but other aspects of the state 
    # are not pushed and popped. So the shuttle's translation 
    # will affect only the light. But the light will shine on 
    # the rest of the scene.
    myTransformSeparator = SoTransformSeparator()
    root.addChild(myTransformSeparator)

    # A shuttle node translates back and forth between the two
    # fields translation0 and translation1.  
    # This moves the light.
    myShuttle = SoShuttle()
    myTransformSeparator.addChild(myShuttle)
    myShuttle.translation0 = (-2, -1, 3)
    myShuttle.translation1 = ( 1,  2, -3)

    # Add the point light below the transformSeparator
    myPointLight = SoPointLight()
    myTransformSeparator.addChild(myPointLight)
    myPointLight.color = (0, 1, 0)

    root.addChild(SoCone())

    myViewer = SoGuiExaminerViewer(myWindow)
    myViewer.setSceneGraph(root)
    myViewer.setTitle("Lights")
    myViewer.setHeadlight(FALSE)
    myViewer.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

###
# Copyright (c) 2002-2007 Systems in Motion
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

###
# This is an example from the Inventor Mentor
# chapter 13, example 7.
#
# A calculator engine computes a closed, planar curve.
# The output from the engine is connected to the translation
# applied to a flower object, which consequently moves
# along the path of the curve.
#

import sys

from pivy.coin import *
from pivy.sogui import *

def main():
    # Initialize Inventor and Qt
    myWindow = SoGui.init(sys.argv[0])  
    if myWindow == None: sys.exit(1)     

    root = SoSeparator()

    # Add a camera and light
    myCamera = SoPerspectiveCamera()
    myCamera.position = (-0.5, -3.0, 19.0)
    myCamera.nearDistance = 10.0
    myCamera.farDistance = 26.0
    root.addChild(myCamera)
    root.addChild(SoDirectionalLight())

    # Rotate scene slightly to get better view
    globalRotXYZ = SoRotationXYZ()
    globalRotXYZ.axis = SoRotationXYZ.X
    globalRotXYZ.angle = M_PI/7
    root.addChild(globalRotXYZ)

    # Read the background path from a file and add to the group
    myInput = SoInput()
    if not myInput.openFile("flowerPath.iv"):
        sys.exit(1)
    flowerPath = SoDB.readAll(myInput)
    if flowerPath == None: sys.exit(1)
    root.addChild(flowerPath)

#############################################################
# CODE FOR The Inventor Mentor STARTS HERE  

    # Flower group
    flowerGroup = SoSeparator()
    root.addChild(flowerGroup)

    # Read the flower object from a file and add to the group
    if not myInput.openFile("flower.iv"):
        sys.exit(1)
    flower = SoDB.readAll(myInput)
    if flower == None: sys.exit(1)

    # Set up the flower transformations
    danceTranslation = SoTranslation()
    initialTransform = SoTransform()
    flowerGroup.addChild(danceTranslation)
    initialTransform.scaleFactor = (10., 10., 10.)
    initialTransform.translation = (0., 0., 5.)
    flowerGroup.addChild(initialTransform)
    flowerGroup.addChild(flower)

    # Set up an engine to calculate the motion path:
    # r = 5*cos(5*theta) x = r*cos(theta) z = r*sin(theta)
    # Theta is incremented using a time counter engine,
    # and converted to radians using an expression in
    # the calculator engine.
    calcXZ = SoCalculator()
    thetaCounter = SoTimeCounter()

    thetaCounter.max = 360
    thetaCounter.step = 4
    thetaCounter.frequency = 0.075

    calcXZ.a.connectFrom(thetaCounter.output)    
    calcXZ.expression.set1Value(0, "ta=a*M_PI/180") # theta
    calcXZ.expression.set1Value(1, "tb=5*cos(5*ta)") # r
    calcXZ.expression.set1Value(2, "td=tb*cos(ta)") # x 
    calcXZ.expression.set1Value(3, "te=tb*sin(ta)") # z 
    calcXZ.expression.set1Value(4, "oA=vec3f(td,0,te)") 
    danceTranslation.translation.connectFrom(calcXZ.oA)

# CODE FOR The Inventor Mentor ENDS HERE
#############################################################

    myRenderArea = SoGuiRenderArea(myWindow)
    myRenderArea.setSceneGraph(root)
    myRenderArea.setTitle("Flower Dance")
    myRenderArea.show()

    SoGui.show(myWindow)
    SoGui.mainLoop()

if __name__ == "__main__":
    main()
